SystemOrganization addCategory: #'SOUL-Tests'!

Object subclass: #LogicTestNotifier
	instanceVariableNames: 'outputStream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOUL-Tests'!

!LogicTestNotifier commentStamp: '<historical>' prior: 0!
Timing information can be gathered while running the QSoul unit tests. This class implements the timing notifications that are sent by the tests (by virtue of the class QSOULLogicTests). Subclasses can override this to calculate sums, display information, save information, etc.!

!LogicTestNotifier class methodsFor: 'Instance Creation' stamp: 'sp 24/10/11 15:26'!
outputStream: aStream
	"Set the outputStream to use. When none is provided; the Transcript is used."

	
	^self new outputStream: aStream! !

!LogicTestNotifier class methodsFor: 'Public' stamp: 'sp 24/10/11 15:26'!
useAsTimer
	"Set the current Timer as the current one to use by LogicTests."

	
	LogicTests setLogicTimer: self new! !

!LogicTestNotifier methodsFor: 'Test Interface' stamp: 'sp 24/10/11 15:26'!
afterCaseDo: aTestCase
	"Action to take after the case is being run."

	"By default do nothing."
	
	! !

!LogicTestNotifier methodsFor: 'Test Interface' stamp: 'sp 24/10/11 15:26'!
afterQuery: results description: descriptionString
	"A query was run, with as description the given string. The results are in 'results'."

	"By default do nothing."
	
	! !

!LogicTestNotifier methodsFor: 'Test Interface' stamp: 'sp 24/10/11 15:26'!
beforeCaseDo: aTestCase
	"Action to take before the case is being run."

	"By default do nothing."
	
	! !

!LogicTestNotifier methodsFor: 'Test Interface' stamp: 'sp 24/10/11 15:26'!
beforeQuery: aString
	"A query will be run (with the code given by aString)."

	"By default do nothing."
	
	! !

!LogicTestNotifier methodsFor: 'Private-accessing' stamp: 'sp 24/10/11 15:26'!
outputStream
	
	outputStream isNil ifTrue: [outputStream := Transcript].
	^outputStream! !

!LogicTestNotifier methodsFor: 'Private-accessing' stamp: 'sp 24/10/11 15:26'!
outputStream: aStream
	
	outputStream := aStream! !

LogicTestNotifier subclass: #TotalTimer
	instanceVariableNames: 'totalsForClasses caseCount'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOUL-Tests'!

!TotalTimer methodsFor: 'Private' stamp: 'sp 24/10/11 15:26'!
addToCurrentCaseCount: number
	
	caseCount = 0
		ifTrue: [caseCount := number]
		ifFalse: [caseCount := caseCount + number]! !

!TotalTimer methodsFor: 'Test Interface' stamp: 'sp 24/10/11 15:26'!
afterCaseDo: aTestCase
	"a testcase was run. As a result, the queries contained in the testcasewere run, and their totals collected in #ranQuery:description:. As a side-effect, the instance variable caseCount remembered the sum of all the timings of these queries. Remember the result here."

	
	self
		keepTime: self caseCount
		forClass: aTestCase class.
	self
		output: aTestCase selector printString
		took: self caseCount! !

!TotalTimer methodsFor: 'Test Interface' stamp: 'sp 24/10/11 15:26'!
afterQuery: results description: descriptionString
	"A query was run, and the results are in 'results'. Output the description and the time it took. Remember the time it took for the current case (the total for the case will be the sum for all its individual queries)"

	
	| took |
	took := results timeTaken.
	self addToCurrentCaseCount: took.
	self output: descriptionString took: took! !

!TotalTimer methodsFor: 'Test Interface' stamp: 'sp 24/10/11 15:26'!
beforeCaseDo: aTestCase
	"a testcase will be run. As a result, the queries contained in the testcase will be run, and their totals collected in #ranQuery:description:. As a side-effect, the instance variable caseCount will remember the sum of all the timings of these queries. This total is reset here."

	
	self resetCaseCount! !

!TotalTimer methodsFor: 'Private' stamp: 'sp 24/10/11 15:26'!
caseCount
	"Return the caseCount"

	
	^caseCount! !

!TotalTimer methodsFor: 'Private' stamp: 'sp 24/10/11 15:26'!
keepTime: took forClass: aClass
	"Something was tested in aClass that took a certain time. Keep the total execution time for that class in the dictionary."

	
	| key currentTotalTime |
	key := aClass name.
	currentTotalTime := self totalsForClasses
		at: key
		ifAbsent: [0].
	self totalsForClasses
		at: key
		put: currentTotalTime + took! !

!TotalTimer methodsFor: 'Private' stamp: 'sp 24/10/11 15:26'!
output: aString took: number
	"Output the given string and the number (indicating the time it took for the result to obtain)"

	
	(self outputStream)
		nextPutAll: aString;
		tab;
		nextPutAll: 'took: ';
		tab;
		print: number;
		cr! !

!TotalTimer methodsFor: 'Public' stamp: 'sp 24/10/11 15:26'!
outputClassResults
	
	| sortedAssociations |
	sortedAssociations := self totalsForClasses associations
		asSortedCollection: [:assoc1 :assoc2 | assoc2 < assoc2].
	sortedAssociations
		do:
			[:assoc | 
			self
				output: assoc key
				took: assoc value]! !

!TotalTimer methodsFor: 'Public' stamp: 'sp 24/10/11 15:26'!
outputTotalResult
	
	self
		output: 'Total'
		took: self totalResult! !

!TotalTimer methodsFor: 'Public' stamp: 'sp 24/10/11 15:26'!
ranQuery: results description: descriptionString
	"A query was run, with as description the given string. The results are in 'results'."

	
	self subclassResponsibility! !

!TotalTimer methodsFor: 'Private' stamp: 'sp 24/10/11 15:26'!
resetCaseCount
	"Reset the counter to hold the total for a single case"

	
	caseCount := 0! !

!TotalTimer methodsFor: 'Private' stamp: 'sp 24/10/11 15:26'!
resetTimings
	"reset the information regarding the classes (cannot do this automatically; since testsuites are just collections of individual testCases. Hence I don't know when the testing starts and ends)."

	
	totalsForClasses := nil! !

!TotalTimer methodsFor: 'Public' stamp: 'sp 24/10/11 15:26'!
runCase: aTestCase do: aBlock
	"a testcase is being run. Keep the result for the testCase's class"

	"a testcase is being run. Run the supplied block. As a result, the queries contained in the testcase will be run, and their totals collected in #ranQuery:description:. As a side-effect, the instance variable caseCount will remember the sum of all the timings of these queries. This is the time this case took to execute. As an alternative, we could also simply time the block, which would time the execution of the whole block, including writing the results, etc. While this is more general, the implementation only includes the time for executing without other overhead."

	
	self resetCaseCount.
	aBlock value.
	self
		keepTime: self caseCount
		forClass: aTestCase class! !

!TotalTimer methodsFor: 'Public' stamp: 'sp 24/10/11 15:26'!
totalResult
	
	^self totalsForClasses values
		inject: 0
		into: [:subTotal :number | subTotal + number]! !

!TotalTimer methodsFor: 'Private' stamp: 'sp 24/10/11 15:26'!
totalsForClasses
	
	totalsForClasses isNil ifTrue: [totalsForClasses := Dictionary new].
	^totalsForClasses! !

TestCase subclass: #LogicTests
	instanceVariableNames: 'currentTotal'
	classVariableNames: 'IsDetailedTimed IsSelectorTimed LogicTimer'
	poolDictionaries: ''
	category: 'SOUL-Tests'!

LogicTests subclass: #ArithmeticTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOUL-Tests'!

!ArithmeticTest methodsFor: 'basic arithmetic' stamp: 'sp 24/10/11 15:26'!
testadd
	"add/3"

	
	self processQuery:
			' if succeeds(and(add([8],[9],?resulta),equals(?resulta,[17])))'
		description: 'Is 8 + 9 = 17 ?'.
	self processQuery: ' if succeeds(and(add(?n1,[2],[-1]),equals(?n1,[-3])))'
		description: 'Is -3 the solution of the equation x + 2 = -1 ?'.
	self processQuery: ' if succeeds(and(add([3],?m1,[5]),equals(?m1,[2])))'
		description: 'Is 2 the solution of the equation 3 + x = 5 ?'.
	self processQuery: ' if succeeds(add([4],[5],[9]))' description:
			'Is 4 + 5 = 9 ?'.
	self processQuery: ' if succeeds(add([-1],[-2],[-3]))' description:
			'Is -1 + -2 = -3 ?'.
	self processQuery: ' if fails(add(?N,?M,?R))' description:
			'Does addition fail when all arguments are unbound?'.
	self processQuery: ' if fails(add([4],[5],[10]))' description:
			'Is 4 + 5 not equal to 10?'! !

!ArithmeticTest methodsFor: 'basic arithmetic' stamp: 'sp 24/10/11 15:26'!
testadd1
	"add1/2"

	
	self processQuery: ' if succeeds(add1([8],[9]))' description:
			'Is 8 + 1 = 9 ?'.
	self processQuery: ' if fails(add1([9],[11]))' description:
			'Is 9 + 1 not equal to 11 ?'.
	self processQuery: ' if succeeds(and(add1(?n1,[2]),equals(?n1,[1])))'
		description: 'Is x = 1 if x + 1 = 2 ?'.
	self processQuery: ' if succeeds(and(add1([3],?m1),equals(?m1,[4])))'
		description: 'Is x = 4 if 3 + 1 = x ?'! !

!ArithmeticTest methodsFor: 'comparison' stamp: 'sp 24/10/11 15:26'!
testcomparable
	"comparable/2"

	
	self processQuery: ' if fails(comparable([Object],[1]))' description:
			'Is Object not comparable with a number ?'.
	self processQuery: ' if succeeds(comparable([2],[1]))' description:
			'Is an integer comparable with another integer ?'.
	self processQuery: ' if succeeds(comparable([2.0],[1]))' description:
			'Is an integer comparable with a real ?'! !

!ArithmeticTest methodsFor: 'comparison' stamp: 'sp 24/10/11 15:26'!
testgreater
	"greater/2"

	
	self processQuery: ' if succeeds(greater([6],[5]))' description:
			'Is 6 greater than 5 ?'.
	self processQuery: ' if fails(greater([5],[6]))' description:
			'Is 5 not greater than 6 ?'.
	self processQuery: ' if fails(greater([3],[3]))' description:
			'Is 3 not strict greater than 3 ?'.
	self processQuery: ' if fails(greater(?N,[5]))' description:
			'Does greater/2 fail when first argument is unbound?'.
	self processQuery: ' if fails(greater([4],?M))' description:
			'Does greater/2 fail when second argument is unbound?'.
	self processQuery: ' if fails(greater(?N,?M))' description:
			'Does greater/2 fail when both arguments are unbound?'! !

!ArithmeticTest methodsFor: 'comparison' stamp: 'sp 24/10/11 15:26'!
testgreaterOrEqual
	"greaterOrEqual/2"

	
	self processQuery: ' if succeeds(greaterOrEqual([6],[5]))' description:
			'Is 6 greater than or equal to 5 ?'.
	self processQuery: ' if succeeds(greaterOrEqual([3],[3]))' description:
			'Is 3 greater than or equal to 3 ?'.
	self processQuery: ' if fails(greaterOrEqual([5],[6]))' description:
			'Is 5 not greater than or equal to 6 ?'.
	self processQuery: ' if fails(greaterOrEqual(?N,[5]))' description:
			'Does greaterOrEqual/2 fail when first argument is unbound?'.
	self processQuery: ' if fails(greaterOrEqual([4],?M))' description:
			'Does greaterOrEqual/2 fail when second argument is unbound?'.
	self processQuery: ' if fails(greaterOrEqual(?N,?M))' description:
			'Does greaterOrEqual/2 fail when both arguments are unbound?'.
	self processQuery: ' if succeeds(greaterOrEqual([5.0],[5]))' description:
			'Is 5.0 greater than or equal to 5 ?'! !

!ArithmeticTest methodsFor: 'type checking' stamp: 'sp 24/10/11 15:26'!
testinteger
	"integer/1"

	
	self processQuery: ' if succeeds(integer([4]))' description:
			'Is the Smalltalk object 4 an integer?'.
	self processQuery: ' if succeeds(integer(4))' description:
			'Is the logic term 4 an integer?'.
	self processQuery: ' if fails(integer(?N))' description:
			'Does integer/1 fail when called with a variable?'.
	self processQuery: ' if fails(integer(bla))' description:
			'Is a logic term representing a symbol not an integer?'.
	self processQuery: ' if fails(integer([''bla'']))' description:
			'Is a Smalltalk string not an integer?'.
	self processQuery: ' if fails(integer([4.0]))' description:
			'Is 4.0 not an integer?'.
	self processQuery: ' if fails(integer([4.2]))' description:
			'Is 4.2 not an integer?'! !

!ArithmeticTest methodsFor: 'type checking' stamp: 'sp 24/10/11 15:26'!
testnumber
	"number/1"

	
	self processQuery: ' if succeeds(number([4]))' description:
			'Is the Smalltalk object 4 a number?'.
	self processQuery: ' if succeeds(integer(4))' description:
			'Is the logic term 4 a number?'.
	self processQuery: ' if fails(number(?N))' description:
			'Does number/1 fail when called with a variable?'.
	self processQuery: ' if fails(number(bla))' description:
			'Is a logic term representing a symbol not a number?'.
	self processQuery: ' if fails(integer([''bla'']))' description:
			'Is a Smalltalk string not an integer?'.
	self processQuery: ' if succeeds(number([4.0]))' description:
			'Is 4.0 a number?'.
	self processQuery: ' if succeeds(number([-21.54]))' description:
			'Is -21.54 a number?'! !

!ArithmeticTest methodsFor: 'type checking' stamp: 'sp 24/10/11 15:26'!
testpositiveInteger
	"positiveInteger/1"

	
	self processQuery: ' if succeeds(positiveInteger([4]))' description:
			'Is the Smalltalk object 4 a positive integer?'.
	self processQuery: ' if succeeds(positiveInteger([0]))' description:
			'Is the Smalltalk object 0 a positive integer?'.
	self processQuery: ' if fails(positiveInteger([-2]))' description:
			'Is the Smalltalk object -2 NOT a positive integer?'.
	self processQuery: ' if succeeds(positiveInteger(4))' description:
			'Is the logic term 4 a positive integer?'.
	self processQuery: ' if fails(positiveInteger(?N))' description:
			'Does positiveInteger/1 fail when called with a variable?'.
	self processQuery: ' if fails(positiveInteger(bla))' description:
			'Is a logic term representing a symbol not a positive integer?'.
	self processQuery: ' if fails(positiveInteger([''bla'']))' description:
			'Is a Smalltalk string not a positive integer?'.
	self processQuery: ' if fails(positiveInteger([4.0]))' description:
			'Is 4.0 not a positive integer?'.
	self processQuery: ' if fails(positiveInteger([4.2]))' description:
			'Is 4.2 not a positive integer?'! !

!ArithmeticTest methodsFor: 'basic arithmetic' stamp: 'sp 24/10/11 15:26'!
testproduct
	"self run:#testproduct"

	"product/3"

	
	self processQuery: ' if succeeds(product([4],[3],[12]))' description:
			'Is 4 * 3 = 12 ?'.
	self processQuery:
			' if succeeds(and(product(?N,[3],[12]),equals(?N,[4])))' description:
			'Is 4 the solution of the equation x * 3 = 12 ?'.
	self processQuery:
			' if succeeds(and(product([4],?N,[12]),equals(?N,[3])))' description:
			'Is 3 the solution of the equation 4 * x = 12 ?'.
	self processQuery:
			' if succeeds(and(product([4],[3],?M),equals(?M,[12])))' description:
			'Is x = 12 if 4 * 3 = 12 ?'.
	self processQuery: ' if fails(product([4],[3],[2]))' description:
			'Is 4 * 3 not equal to 2?'.
	self processQuery: ' if fails(product(?N,?M,?R))' description:
			'Does substraction fail when all arguments are unbound?'! !

!ArithmeticTest methodsFor: 'comparison' stamp: 'sp 24/10/11 15:26'!
testsmaller
	"smaller/2"

	
	self processQuery: ' if succeeds(smaller([5],[6]))' description:
			'Is 5 smaller than 6 ?'.
	self processQuery: ' if fails(smaller([6],[4]))' description:
			'Is 6 not smaller than 4 ?'.
	self processQuery: ' if fails(smaller([3],[3]))' description:
			'Is 3 not strict smaller than 3 ?'.
	self processQuery: ' if fails(smaller(?N,[5]))' description:
			'Does smaller/2 fail when first argument is unbound?'.
	self processQuery: ' if fails(smaller([4],?M))' description:
			'Does smaller/2 fail when second argument is unbound?'.
	self processQuery: ' if fails(smaller(?N,?M))' description:
			'Does smaller/2 fail when both arguments are unbound?'! !

!ArithmeticTest methodsFor: 'comparison' stamp: 'sp 24/10/11 15:26'!
testsmallerOrEqual
	"smallerOrEqual/2"

	
	self processQuery: ' if succeeds(smallerOrEqual([5],[6]))' description:
			'Is 5 smaller than or equal to 6 ?'.
	self processQuery: ' if fails(smallerOrEqual([6],[4]))' description:
			'Is 6 not smaller than or equal to 4 ?'.
	self processQuery: ' if succeeds(smallerOrEqual([5],[5]))' description:
			'Is 5 smaller than or equal to 5 ?'.
	self processQuery: ' if fails(smallerOrEqual(?N,[5]))' description:
			'Does smallerOrEqual/2 fail when first argument is unbound?'.
	self processQuery: ' if fails(smallerOrEqual([4],?M))' description:
			'Does smallerOrEqual/2 fail when second argument is unbound?'.
	self processQuery: ' if fails(smallerOrEqual(?N,?M))' description:
			'Does smallerOrEqual/2 fail when both arguments are unbound?'.
	self processQuery: ' if succeeds(smallerOrEqual([5.0],[5]))' description:
			'Is 5.0 smaller than or equal to 5 ?'! !

!ArithmeticTest methodsFor: 'type checking' stamp: 'sp 24/10/11 15:26'!
teststrictPositiveInteger
	"positiveInteger/1"

	
	self processQuery: ' if succeeds(strictPositiveInteger([4]))' description:
			'Is the Smalltalk object 4 a strict positive integer?'.
	self processQuery: ' if fails(strictPositiveInteger([0]))' description:
			'Is the Smalltalk object 0 NOT a strict positive integer?'.
	self processQuery: ' if fails(strictPositiveInteger(0))' description:
			'Is the logic constant 0 NOT a strict positive integer?'.
	self processQuery: ' if fails(strictPositiveInteger([-2]))' description:
			'Is the Smalltalk object -2 NOT a strict positive integer?'.
	self processQuery: ' if succeeds(strictPositiveInteger(4))' description:
			'Is the logic term 4 a strict positive integer?'.
	self processQuery: ' if fails(strictPositiveInteger(?N))' description:
			'Does strictPositiveInteger/1 fail when called with a variable?'.
	self processQuery: ' if fails(strictPositiveInteger(bla))' description:
			'Is a logic term representing a symbol not a strict positive integer?'.
	self processQuery: ' if fails(strictPositiveInteger([''bla'']))'
		description: 'Is a Smalltalk string not a strict positive integer?'.
	self processQuery: ' if fails(strictPositiveInteger([4.0]))' description:
			'Is 4.0 not a strict positive integer?'.
	self processQuery: ' if fails(strictPositiveInteger([4.2]))' description:
			'Is 4.2 not a strict positive integer?'! !

!ArithmeticTest methodsFor: 'basic arithmetic' stamp: 'sp 24/10/11 15:26'!
testsub
	"sub/3"

	
	self processQuery: ' if succeeds(sub([4],[3],[1]))' description:
			'Is 4 - 3 = 1 ?'.
	self processQuery: ' if succeeds(and(sub(?N1,[3],[1]),equals(?N1,[4])))'
		description: 'Is 4 the solution of the equation x - 3 = 1 ?'.
	self processQuery: ' if succeeds(and(sub([4],?N2,[1]),equals(?N2,[3])))'
		description: 'Is 3 the solution of the equation 4 - x = 1 ?'.
	self processQuery: ' if succeeds(and(sub([4],[3],?M),equals(?M,[1])))'
		description: 'Is x = 1 if 4 - 3 = x ?'.
	self processQuery: ' if fails(sub([4],[3],[2]))' description:
			'Is 4 - 3 not equal to 2?'.
	self processQuery: ' if fails(sub(?N,?M,?R))' description:
			'Does substraction fail when all arguments are unbound?'! !

!ArithmeticTest methodsFor: 'basic arithmetic' stamp: 'sp 24/10/11 15:26'!
testsub1
	"sub1/2"

	
	self processQuery: ' if succeeds(sub1([4],[3]))' description:
			'Is 4 - 1 = 3 ?'.
	self processQuery: ' if fails(sub1([4],[2]))' description:
			'Is 4 - 1 not equal to 2 ?'.
	self processQuery: ' if succeeds(and(sub1(?N1,[3]),equals(?N1,[4])))'
		description: 'Is x = 4 if x - 1 = 3 ?'.
	self processQuery: ' if succeeds(and(sub1([4],?N2),equals(?N2,[3])))'
		description: 'Is x = 3 if 4 - 1 = x ?'! !

!ArithmeticTest methodsFor: 'comparison' stamp: 'sp 24/10/11 15:26'!
testzero
	"zero/1"

	
	self processQuery: ' if succeeds(zero([0]))' description:
			'Is the Smalltalk integer 0 zero?'.
	self processQuery: ' if fails(zero([4]))' description:
			'Is the Smalltalk number 4 not zero?'.
	self processQuery: ' if succeeds(zero(0))' description:
			'Is the logic constant 0 zero?'.
	self processQuery: ' if fails(zero(4))' description:
			'Is the logic constant 4 not zero?'.
	self processQuery: ' if succeeds(and(zero(?N),equals(?N,[0])))'
		description: 'Is 0 filled in when zero/1 is called with a variable?'! !

LogicTests subclass: #DataHandlingTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOUL-Tests'!

!DataHandlingTest methodsFor: 'list handling' stamp: 'sp 24/10/11 15:26'!
testappend
	"self run:#testappend"

	
	self processQuery:
			' if succeeds(and(append(<[1],[2],[3]>,<[4],[5],[6]>,?r),equals(?r,<[1],[2],[3],[4],[5],[6]>)))'
		description: 'Can we append two simple lists?'.
	self processQuery:
			' if succeeds(append(<[1],[2],[3]>,<[4],[5],[6]>,<[1],[2],[3],[4],[5],[6]>))'
		description: 'Is the append of two simple lists the appended list?'.
	self processQuery:
			' if succeeds(and(append(<[1],[2],[3]>,?S,<[1],[2],[3],[4],[5],[6]>),equals(?S,<[4],[5],[6]>)))'
		description:
			'Can we deduce what second string needs be appended to a first one to get a certain result?'.
	self processQuery:
			' if succeeds(and(append(?F,<[3],[4],[5],[6]>,<[1],[2],[3],[4],[5],[6]>),equals(?S,<[1],[2]>)))'
		description:
			'Can we deduce what first string needs be appended to a second one to get a certain result?'.
	self processQuery:
			' if findall(?F,append(?F,?S,<[1],[2],[3],[4]>),<<>,<[1]>,<[1],[2]>,<[1],[2],[3]>,<[1],[2],[3],[4]>>)'
		description:
			'What possible strings can be appended to get a certain result string?'! !

!DataHandlingTest methodsFor: 'string handling' stamp: 'sp 24/10/11 15:26'!
testappendAtoms
	"self run:#testappendAtoms"

	"appendAtoms/3"

	
	self processQuery: ' if succeeds(appendAtoms(a,b,ab))' description:
			'Does appendAtoms/3 work correctly with 3 instantiated arguments?'.
	self processQuery: ' if succeeds(and(appendAtoms(a,b,?c),equals(?c,ab)))'
		description:
			'Does appendAtoms/3 work correctly with only first 2 arguments instantiated?'.
	self processQuery: ' if succeeds(appendAtoms([''a''],[''b''],[#ab]))'
		description:
			'The two first arguments may be strings, but the third is always a symbol.'.
	self processQuery:
			' if succeeds(and(appendAtoms([''a''],[''b''],?c),equals(?c,ab)))'
		description:
			'The two first arguments may be strings, but the third is always a symbol.'	"TODO: Diabled the tests below because in Squeak 'ab' = #'ab' is true, hence strings and symbols unify and the tests below fail. Should we fix the tests or the behavior of the predicate on Squeak? Fixing the behavior might be better as there is now the strange situation that the predicate can be true when the last argument is a string, but if the last argument is left unbound, no string will be given as a solution."
	"
	self	processQuery: 'if fails(appendAtoms([''a''],[''b''],[''ab'']))'
		description: 'As the third is assumed to be a symbol, the predicate will fail when it is a string.'.

	self	processQuery: 'if fails(and(appendAtoms([''a''],[''b''],?c),equals(?c,[''ab''])))'
		description: 'As the third is assumed to be a symbol, the predicate will fail when it is a string.'.
"! !

!DataHandlingTest methodsFor: 'the clause database' stamp: 'sp 24/10/11 15:26'!
testassert1
	"self run:#testassert1"

	
	self processQuery: ' if assert(foo(foo(blabla))),foo(foo(blabla))'
		description: 'Does assert work properly for simple facts?'.
	self processQuery: ' if assert(foo([Object])),foo([Object])' description:
			'Does assert work properly for facts containing a simple smalltalkterm?'.
	self processQuery: ' if equals(?x,[Class]),assert(foo(?x)),foo([Class])'
		description:
			'Does assert work properly for facts containing a smalltalkterm with variables?'.
	self repository retractAll: 'foo/1'	"What about using assert/1 on logic rules? Which format to use?"! !

!DataHandlingTest methodsFor: 'the clause database' stamp: 'sp 24/10/11 15:26'!
testassert2
	"self run:#testassert2"

	
	self processQuery: ' if getRepository(?rep),assert(foo(x)),foo(x)'
		description:
			'Check if the assert can assert a clause in the repository when it is given as a parameter'.
	self repository retractAll: 'foo/1'! !

!DataHandlingTest methodsFor: 'term type checking' stamp: 'sp 24/10/11 15:26'!
testatom
	"self run:#testatom"

	"atom/1"

	
	self processQuery: ' if fails(atom(?X))' description:
			'Does atom/1 fail for a variable?'.
	self processQuery: ' if succeeds(atom(<>))' description:
			'Does atom/1 succeed for a non-variable such as an empty list?'.
	self processQuery: ' if fails(atom(<a,b,?x,d>))' description:
			'Does atom/1 fail for a list?'.
	self processQuery: ' if succeeds(atom(hallo))' description:
			'Does atom/1 succeed for a symbol?'.
	self processQuery: ' if succeeds(atom(5))' description:
			'Does atom/1 succeed for a number?'.
	self processQuery: ' if succeeds(atom([''hallo'']))' description:
			'Does atom/1 succeed for a string?'.
	self processQuery: ' if succeeds(atom([Object]))' description:
			'Does atom/1 succeed for a Smalltalk class?'.
	self processQuery: ' if fails(atom(hallo(foo,bar)))' description:
			'Does atom/1 fail for a compound term?'.
	self processQuery: ' if fails(atom(hallo(?X,bar)))' description:
			'Does atom/1 fail for a compound term containing a variable?'! !

!DataHandlingTest methodsFor: 'pattern matching' stamp: 'sp 24/10/11 15:26'!
testatomToChars
	"self run:#testatomToChars"

	
	self processQuery: ' if succeeds(atomToChars(hello,?x))' description:
			'Generate a list of characters from a given atom'.
	self processQuery:
			' if succeeds(atomToChars(hello,<[#h],[#e],[#l],[#l],[#o]>))'
		description:
			'I dont understand why this one gives an error. There is something weird going on here...'.
	self processQuery: ' if succeeds(atomToChars([#hello],?x))' description:
			'Generate a list of characters from a given atom'.
	self processQuery:
			' if succeeds(atomToChars(hello,<[#h],[#e],[#l],[#l],[#o]>))'
		description: 'Does this also work when all arguments are bound?'.
	self processQuery: ' if fails(atomToChars(?atom,?x))' description:
			'Predicate fails when ?atom is not bound'.
	self processQuery:
			' if fails(atomToChars(?atom,<[#h],[#e],[#l],[#l],[#o]>))' description:
			'Predicate fails when ?atom is not bound'! !

!DataHandlingTest methodsFor: 'term type checking' stamp: 'sp 24/10/11 15:26'!
testcompound
	"self run:#testcompound"

	
	self processQuery: ' if compound(woman(maria))' description:
			'A simple compound'.
	self processQuery: ' if compound(woman(?x))' description:
			'A compound with variables'.
	self processQuery: ' if compound(<a,b,c>)' description:
			'A lists is a compound'.
	self processQuery: ' if compound(<kim>)' description:
			'A cons-pair is also a compound'.
	self processQuery: ' if fails(compound(?X))' description:
			'A variable is not a compound'.
	self processQuery: ' if fails(compound(hallo))' description:
			'An atom is not a compound'.
	self processQuery: ' if fails(compound(<>))' description:
			'An empty list is not a compound'! !

!DataHandlingTest methodsFor: 'quoted term handling' stamp: 'sp 24/10/11 15:26'!
testconcatQuotedTerms
	"self run:#testconcatQuotedTerms"

	"concatQuotedTerms/3"

	
	self processQuery: ' if succeeds(concatQuotedTerms(<>,{ },{}))'
		description:
			'Is the empty list converted in an empty quoted code term (regardless of the separator)?'.
	self processQuery: ' if succeeds(concatQuotedTerms(<{foo}>,{ },{foo}))'
		description:
			'Is a single quoted code term converted into itself (regardless of the separator)?'.
	self processQuery:
			' if succeeds(and(concatQuotedTerms(<{foo},{bar}>,{ },?result),equals(?result,{foo bar})))'
		description:
			'Is a list of two quoted code terms converted into a quoted code term containing those two quoted code terms with a space in between (when the separator is a space)?'.
	self processQuery:
			' if succeeds(and(concatQuotedTerms(<{foo},{bar},{zork}>,{ },?result),equals(?result,{foo bar zork})))'
		description:
			'Is a list of three quoted code terms converted into a quoted code term containing those three quoted code terms with a space between each two (when the separator is a space)?'! !

!DataHandlingTest methodsFor: 'list handling' stamp: 'sp 24/10/11 15:26'!
testdeepflatten
	"deepflatten/2"

	
	self processQuery:
			' if succeeds(deepflatten(<<[1],<[2]>,[3]>,<<<[4]>,[5]>,[6]>>,<[1],[2],[3],[4],[5],[6]>))'
		description:
			'Can we test whether a shallow list is a flattened version of a deeply nested list?'.
	self processQuery:
			' if succeeds(and(deepflatten(<<[1],[2],[3]>,<[4],[5],[6]>>,?flatlist),equals(?flatlist,<[1],[2],[3],[4],[5],[6]>)))'
		description: 'Can we flatten a deeply nested list into a shallow one?'.
	self processQuery:
			' if succeeds(and(deepflatten(<<[1],<hallo,aloha>,[2],<>,[3]>,<[4],<>,[5],[6]>>,?flatlist),equals(?flatlist,<[1],hallo,aloha,[2],[3],[4],[5],[6]>)))'
		description:
			'Can we flatten a deeply nested list that contains empty sublists into a shallow one?'.
	self processQuery: ' if fails(deepflatten(?x,<a,b,c,d>))' description:
			'Does deepflatten/2 fail when the first argument is variable?'! !

!DataHandlingTest methodsFor: 'set handling operators' stamp: 'sp 24/10/11 15:26'!
testdifference
	"difference/3"

	
	self processQuery:
			' if succeeds(difference(<a,e,y,f,u,r,d>,<e,k,f,r,u>,?x))' description:
			'Can the difference between two sets be computed?'.
	self processQuery:
			' if succeeds(and(difference(<a,e,y,f,u,r,d>,<e,k,f,r,u>,?x),equals(?x,<a,y,d>)))'
		description: 'Is the difference between two sets computed correctly?'.
	self processQuery:
			' if succeeds(difference(<a,e,y,f,u,r,d>,<e,k,f,r,u>,<a,y,d>))'
		description: 'Is the difference between two sets computed correctly?'.
	self processQuery:
			' if succeeds(difference(<a,e,y,f,u,r,d>,<g,k,h,m,s>,<a,e,y,f,u,r,d>))'
		description: 'Computing a difference for an empty intersection.'! !

!DataHandlingTest methodsFor: 'term comparison and sorting' stamp: 'sp 24/10/11 15:26'!
testdiffers
	"differs/2"

	
	self processQuery: ' if fails(differs(?X,?Y))' description:
			'Are two unbound logic variables NOT different?'.
	self processQuery: ' if fails(differs([1],[1]))' description:
			'Are two equal integers not different?'.
	self processQuery: ' if succeeds(differs([1],[2]))' description:
			'Are two unequal integers different?'.
	self processQuery: ' if fails(differs([1],1))' description:
			'Are the Smalltalk integer 1 and logic constant 1 not different?'.
	self processQuery: ' if fails(differs([#test],test))' description:
			'Are a Smalltalk symbol and its corresponding logic term not different?'! !

!DataHandlingTest methodsFor: 'set handling checks' stamp: 'sp 24/10/11 15:26'!
testdisjoint
	"disjoint/2"

	
	self processQuery: ' if succeeds(disjoint(<a,b,c>,<d,g,f,h>))'
		description: 'Are two sets disjoint?'.
	self processQuery: ' if succeeds(disjoint(<>,<d,g,f,h>))' description:
			'Is the empty set disjoint with another one?'.
	self processQuery: ' if succeeds(disjoint(<a,b,c>,<>))' description:
			'Is a set disjoint with the empty set?'.
	self processQuery: ' if succeeds(disjoint(<>,<>))' description:
			'Are two empty sets disjoint?'.
	self processQuery: ' if fails(disjoint(<a,b,c>,<d,e,b,f>))' description:
			'Are two sets not disjoint?'! !

!DataHandlingTest methodsFor: 'list handling' stamp: 'sp 24/10/11 15:26'!
testemptyList
	"emptyList/1"

	
	self processQuery: ' if succeeds(emptyList(<>))' description:
			'Is <> the notation for an empty list?'.
	self processQuery: ' if fails(emptyList(<1,2,3>))' description:
			'Is <1,2,3> not an empty list?'.
	self processQuery: ' if succeeds(and(emptyList(?var),equals(?var,<>)))'
		description:
			'When calling emptyList with a variable, is the variable bound to <>?'.
	self processQuery:
			' if succeeds(and(equals(<a>,?x),equals(?x,<?y|?z>),emptyList(?z)))'
		description: 'Is the tail of a singleton-list empty?'! !

!DataHandlingTest methodsFor: 'string handling' stamp: 'sp 24/10/11 15:26'!
testendsWith
	"self run: #testendsWith"

	"endsWith/2"

	
	self processQuery: ' if succeeds(endsWith([''aTest''],[''est'']))'
		description: 'Does a certain string end with a certain postfix?'.
	self processQuery: ' if succeeds(endsWith([''aTest''],st))' description:
			'Does a certain string end with a certain postfix?'.
	self processQuery: ' if succeeds(endsWith(aTest,[''Test'']))' description:
			'Does a certain string end with a certain postfix?'.
	self processQuery: ' if succeeds(endsWith(aTest,Test))' description:
			'Does a certain string end with a certain postfix?'.
	self processQuery: ' if succeeds(endsWith(aTest,aTest))' description:
			'Does a string have itself as postfix?'.
	self processQuery: ' if succeeds(endsWith(aTest,['''']))' description:
			'Does a certain string end with the empty postfix?'.
	self processQuery: ' if succeeds(endsWith(aTest,atest))' description:
			'Is the endsWith predicate case INsensitive?'.
	self processQuery: ' if succeeds(endsWith([''aTest''],[''atest'']))'
		description: 'Is the endsWith predicate case INsensitive?'! !

!DataHandlingTest methodsFor: 'term comparison and sorting' stamp: 'sp 24/10/11 15:26'!
testequals
	"equals/2"

	
	self processQuery: ' if succeeds(equals([#add:],[#add:]))' description:
			'Are two equal Smalltalk symbols the same?'.
	self processQuery: ' if succeeds(equals([#test],test))' description:
			'Are a Smalltalk symbol and its corresponding logic term equal?'.
	self processQuery: ' if succeeds(equals([1],[1.0]))' description:
			'Is the integer 1 equal to the real number 1.0?'.
	self processQuery: ' if succeeds(and(equals(?X,?Y),var(?X),var(?Y)))'
		description: 'Can a variable be unified with another variable?'.
	self processQuery:
			' if succeeds(and(equals(?X,?Y),equals(?X,1),equals(?Y,1)))'
		description:
			'After unifying two variables and assigning a value to one of them, does the other one automatically get bound to the same value?'.
	self processQuery: ' if succeeds(equals(?X,[#add:]))' description:
			'Can a variable be unified with a value?'.
	self processQuery: ' if succeeds(equals([#add:],?Y))' description:
			'Can a value be unified with a variable?'.
	self processQuery: ' if succeeds(equals([#add],add))' description:
			'Is a Smalltalk symbol equal to the corresponding logic constant?'.
	self processQuery: ' if succeeds(equals(add,add))' description:
			'Are two equal logic constants (symbols) unified?'.
	self processQuery: ' if succeeds(equals(1,1))' description:
			'Are two equal logic constants (integers) unified?'.
	self processQuery: ' if succeeds(equals([1],1))' description:
			'Is a Smalltalk integer equal to the corresponding logic constant?'.
	self processQuery: ' if succeeds(equals([1.0],1))' description:
			'Is a Smalltalk integer in real form equal to the corresponding logic constant?'.
	self processQuery: ' if fails(equals([#add:],[#add]))' description:
			'Are two different Smalltalk symbols unequal?'.
	self processQuery: ' if succeeds(equals(<kim,tom>,<kim,tom>))'
		description:
			'Are two equal lists equal unregarding their external representation?'.
	self processQuery: ' if fails(equals(<kim,tom>,<tom,kim>))' description:
			'Are two unequal lists with different representation unequal?'.
	self processQuery: ' if fails(equals(<kim,tom>,<tom,kim>))' description:
			'Are two unequal lists with same representation unequal?'.
	self processQuery:
			' if succeeds(and(equals(?X,<x,y,z,u,v>),equals(<x,y,z,u,v>,?X)))'
		description: 'Can a variable be unified with a list?'.
	self processQuery: ' if succeeds(equals(<x,y,z,u,v>,<x,y,z,u,v>))'
		description:
			'Is a list in shorthand notation equal to its expanded form?'! !

!DataHandlingTest methodsFor: 'term conversion' stamp: 'sp 24/10/11 15:26'!
testequalsStructureList
	"self run:#testequalsStructureList"

	"self debug:#testequalsStructureList"

	"equalsStructureList/2"

	
	self processQuery: ' if succeeds(equalsStructureList(foo(bar),?list))'
		description:
			'Does equalsStructureList/2 work when first argument is a fully instantiated compound term?'.
	self processQuery:
			' if succeeds(and(equalsStructureList(foo(bar),?list),equals(?list,<foo,bar>)))'
		description:
			'Does equalsStructureList/2 produced the correct result when first argument is a fully instantiated compound term?'.
	self processQuery: ' if succeeds(equalsStructureList(foo(bar),<foo,bar>))'
		description:
			'Does equalsStructureList/2 produced the correct result when first argument is a fully instantiated compound term?'.
	self processQuery: ' if succeeds(equalsStructureList(foo(?x),?list))'
		description:
			'Does equalsStructureList/2 work when first argument is a partially instantiated compound term?'.
	self processQuery: ' if succeeds(equalsStructureList(foo(?x),<foo,?x>))'
		description:
			'Does equalsStructureList/2 produced the correct result when first argument is a partially instantiated compound term?'.
	self processQuery: ' if succeeds(equalsStructureList(?x(?y),?list))'
		description:
			'Does equalsStructureList/2 work when first argument is a fully uninstantiated compound term?'.
	self processQuery: ' if succeeds(equalsStructureList(?x(?y),<?x,?y>))'
		description:
			'Does equalsStructureList/2 produced the correct result when first argument is a fully uninstantiated compound term?'.
	self processQuery: ' if succeeds(equalsStructureList(foo,?list))'
		description:
			'Does equalsStructureList/2 work when first argument is a non-compound term?'.
	self processQuery:
			' if succeeds(and(equalsStructureList(foo,?list),equals(?list,<foo>)))'
		description:
			'Does equalsStructureList/2 produce the correct result when first argument is a non-compound term?'.
	self processQuery: ' if succeeds(equalsStructureList(foo,<foo>))'
		description:
			'Does equalsStructureList/2 produce the correct result when first argument is a non-compound term?'.
	self processQuery: ' if fails(equalsStructureList(?x,?l))' description:
			'Does equalsStructureList/2 fails when both arguments are variable?'.
	self processQuery: ' if fails(equalsStructureList(?x,<?x>))' description:
			'Does equalsStructureList/2 fails when both arguments are variable?'.
	self processQuery: ' if fails(equalsStructureList(foo(bar),<foo(bar)>))'
		description:
			'Does equalsStructureList/2 not allow compound terms as keys?'.
	self processQuery:
			' if succeeds(equalsStructureList(foo(bar(a),zork(2),flur(e)),<foo|?More>))'
		description: 'Check correctness of equalsStructureList/2'.
	self processQuery:
			' if succeeds(and(equalsStructureList(foo(bar(a),zork(2),flur(e)),<foo|?More>),equals(?More,<bar(a),zork(2),flur(e)>)))'
		description: 'Check correctness of equalsStructureList/2'.
	self processQuery: ' if succeeds(equalsStructureList(?term,<foo,bar>))'
		description:
			'Does equalsStructureList/2 work when second argument is list with two bound arguments?'.
	self processQuery:
			' if succeeds(and(equalsStructureList(?term,<foo,bar>),equals(?term,foo(bar))))'
		description:
			'Does equalsStructureList/2 produce the correct result when second argument is list with two bound arguments?'.
	self processQuery:
			' if succeeds(and(equalsStructureList(?term,<foo,?y>),equals(?term,foo(?y))))'
		description: 'Check correctness of equalsStructureList/2'	"THESE CASES ARE NOT SUPPORTED YET...

	self	processQuery: 'if succeeds(and(equalsStructureList(?term,<?x,bar>),equals(?term,?x(bar))))'
		description: 'Check correctness of equalsStructureList/2'.

	self	processQuery: 'if succeeds(and(equalsStructureList(?term,<?x,?y>),equals(?term,?x(?y))))'
		description: 'Check correctness of equalsStructureList/2'
"! !

!DataHandlingTest methodsFor: 'list handling' stamp: 'sp 24/10/11 15:26'!
testflatten
	"flatten/2"

	
	self processQuery: ' if succeeds(flatten(<<1,2,3>,<4,5,6>>,?flatlist))'
		description: 'Can we computed the flattened version of a nested list?'.
	self processQuery:
			' if succeeds(flatten(<<1,2,3>,<4,5,6>>,<1,2,3,4,5,6>))' description:
			'Is the flattened version of a nested list computed correctly?'.
	self processQuery:
			' if succeeds(and(flatten(<<[1],[2],[3]>,<[4],[5],[6]>>,?flatlist),equals(?flatlist,<[1],[2],[3],[4],[5],[6]>)))'
		description:
			'Does flatten/2 work correctly with second argument unbound?'.
	self processQuery:
			' if succeeds(flatten(<<1,<2>,3>,<<<4>,5>,6>>,<1,<2>,3,<<4>,5>,6>))'
		description:
			'Is the flattened version of a nested list computed correctly when the list is
					more than one level deep?'.
	self processQuery:
			' if succeeds(flatten(<<1,2,3>,<>,<a,b>,<c>,<>>,<1,2,3,a,b,c>))'
		description:
			'Does flatten work correctly when the list to be flattened contain empty sublists?'! !

!DataHandlingTest methodsFor: 'term type checking' stamp: 'sp 24/10/11 15:26'!
testground
	"self run:#testground"

	
	self processQuery: ' if fails(ground(?X))' description:
			'Is a variable not ground?'.
	self processQuery: ' if fails(ground(?))' description:
			'Is an unnamed variable not ground?'.
	self processQuery: ' if succeeds(ground(<>))' description:
			'Is an empty list ground?'.
	self processQuery: ' if fails(ground(<a,b,?x,d>))' description:
			'Is a list that contains a variable ground?'.
	self processQuery: ' if fails(ground(<a,b,?,d>))' description:
			'Is a list that contains an unnamed variable ground?'.
	self processQuery: ' if succeeds(ground(<a,b,c,d>))' description:
			'Is a fully instantiated list ground?'! !

!DataHandlingTest methodsFor: 'list handling' stamp: 'sp 24/10/11 15:26'!
testhead
	"head/2"

	
	self processQuery: ' if succeeds(head(a,<a,b,c>))' description:
			'Does head/2 work correctly?'.
	self processQuery: ' if succeeds(head(?x,<a,b,c>))' description:
			'Does head/2 work when first argument is unbound?'.
	self processQuery: ' if succeeds(and(head(?x,<a,b,c>),equals(?x,a)))'
		description: 'Does head/2 work correctly when first argument is unbound?'.
	self processQuery: ' if succeeds(head(a,<?x,b,c>))' description:
			'Does head/2 work when first element of list is variable?'.
	self processQuery: ' if succeeds(and(head(a,<?x,b,c>),equals(?x,a)))'
		description:
			'Does head/2 work correctly when first element of list is variable?'.
	self processQuery: ' if succeeds(head(?y,<?x,b,c>))' description:
			'Does head/2 work when first element of list is variable?'.
	self processQuery: ' if succeeds(and(head(?y,<?x,b,c>),equals(?x,?y)))'
		description: 'Does head/2 work when first element of list is variable?'.
	self processQuery: ' if fails(head(?x,<>))' description:
			'Does the empty list have no head?'! !

!DataHandlingTest methodsFor: 'set handling checks' stamp: 'sp 24/10/11 15:26'!
testincludes
	"includes/2"

	
	self processQuery: ' if succeeds(includes(c,<d,a,c,b,f>))' description:
			'Does a certain element belong to a set?'.
	self processQuery: ' if fails(includes(q,<d,a,c,b,f>))' description:
			'Does a certain element not belong to a set?'.
	self
		processQuery: ' if includes(?El,<d,a,c,b,f>)'
		resultsCheck: [:results | results size = 5]
		description: 'Find all elements that belong to some set'.
	self processQuery: ' if fails(includes(q,<>))' description:
			'Does a certain element not belong to the empty set?'.
	self processQuery: ' if fails(includes(?E,<>))' description:
			'Does nothing belong to the empty set?'! !

!DataHandlingTest methodsFor: 'set handling operators' stamp: 'sp 24/10/11 15:26'!
testintersection
	"intersection/3"

	
	self processQuery:
			' if succeeds(intersection(<a,e,y,f,u,r,d>,<e,k,f,r,u>,?x))'
		description: 'Can the intersection of two sets be computed?'.
	self processQuery:
			' if succeeds(and(intersection(<a,e,y,f,u,r,d>,<e,k,f,r,u>,?x),equals(?x,<e,f,u,r>)))'
		description: 'Is the intersection of two sets computed correctly?'.
	self processQuery:
			' if succeeds(intersection(<a,e,y,f,u,r,d>,<e,k,f,r,u>,<e,f,u,r>))'
		description: 'Is the intersection of two sets computed correctly?'.
	self processQuery:
			' if succeeds(intersection(<a,e,y,f,u,r,d>,<g,k,h,m,s>,<>))'
		description: 'Computing an empty intersection.'! !

!DataHandlingTest methodsFor: 'list handling' stamp: 'sp 24/10/11 15:26'!
testintersectionLists
	"self run:#testintersectionLists"

	"Wat zou hier het resultaat van moeten zijn ?!!
		if intersectionLists(<<?a, ?b>,<?a, ?z>>, ?i)"

	
	self processQuery:
			' if succeeds(and(intersectionLists(<<[1],[2],[42]>,<[2],[3],[42]>,<[3],[4],[42]>>,?i),equals(?i,<[42]>)))'
		description: 'Can we find the intersection of three lists?'.
	self processQuery: ' if fails(intersectionLists(<<[1],[2]>,?V>,?i))'
		description:
			'Is it true that intersectionLists cannot contain variables as lists to intersect?'.
	self processQuery:
			' if fails(intersectionLists(<<[1],[2],[42]>,<[2],?,[42]>,<[3],[4],[42]>>,?i))'
		description:
			'Is it true that intersectionLists cannot even contain variables in the lists to intersect?'! !

!DataHandlingTest methodsFor: 'string handling' stamp: 'sp 24/10/11 15:26'!
testistAsSeparatedString
	"self run:#testlistAsSeparatedString"

	"listAsSeparatedString/3"

	
	self processQuery:
			' if succeeds(listAsSeparatedString(<>,['' ''],['''']))' description:
			'Is the empty list converted in an empty string (regardless of the separator)?'.
	self processQuery:
			' if succeeds(listAsSeparatedString(<foo>,['' ''],[''foo'']))'
		description:
			'Is a single symbol converted into a string containing just that symbol (regardless of the separator)?'.
	self processQuery:
			' if succeeds(listAsSeparatedString(<foo,bar>,['' ''],[''foo bar'']))'
		description:
			'Is a list of two symbols converted into a list containing those two symbols with a space in between (when the separator is a space)?'.
	self processQuery:
			' if succeeds(listAsSeparatedString(<foo,bar,zork>,['' ''],[''foo bar zork'']))'
		description:
			'Is a list of three symbols converted into a list containing those three symbols with a space between each two symbols (when the separator is a space)?'! !

!DataHandlingTest methodsFor: 'list handling' stamp: 'sp 24/10/11 15:26'!
testlength
	"self run:#testlength"

	"This default test, which always fails, was generated automatically.
	 Please replace with an adequate test of predicate length!!"

	
	self processQuery: ' if succeeds(length(<[1],[2],[3]>,3))' description:
			'checks whether the length of a list of length 3 is well-calculated'.
	self processQuery:
			' if succeeds(and(length(<[1],[2],[3]>,?x),equals(?x,3)))' description:
			'checks whether the length of a list of length 3 is well-calculated'.
	self processQuery: ' if succeeds(length(<>,0))' description:
			'has the empty list length 0?'.
	self processQuery: ' if length(?list,[5]),equals(?list,<?,?,?,?,?>)'
		description:
			'can the predicate be used to generate a list of a given length?'.
	self processQuery:
			' if not(succeeds(length(?l,3),equals(?l,<?x,?x,?x>),equals(?l,<1,2,3>)))'
		description: 'is the resulting list not filled with anonymous variables?'! !

!DataHandlingTest methodsFor: 'list handling' stamp: 'sp 24/10/11 15:26'!
testlist
	"list/1"

	
	self processQuery: ' if tail(?l,<[1]>),list(?l)' description:
			'Is the empty tail of a list a list ?'.
	self processQuery: ' if tail(?l,<[1],[2],[3]>),list(?l)' description:
			'Is the non-empty tail of a list a list ?'.
	self processQuery: ' if not(list(foo([2])))' description:
			'Is a compound term not a list ?'.
	self processQuery: ' if not(list(?v))' description:
			'Is an unbound variable not a list ?'! !

!DataHandlingTest methodsFor: 'string handling' stamp: 'sp 24/10/11 15:26'!
testlistAsSeparatedString
	"self run:#testlistAsSeparatedString"

	"listAsSeparatedString/3"

	
	self processQuery:
			' if succeeds(listAsSeparatedString(<>,['' ''],['''']))' description:
			'Is the empty list converted in an empty string (regardless of the separator)?'.
	self processQuery:
			' if succeeds(listAsSeparatedString(<foo>,['' ''],[''foo'']))'
		description:
			'Is a single symbol converted into a string containing just that symbol (regardless of the separator)?'.
	self processQuery:
			' if succeeds(listAsSeparatedString(<foo,bar>,['' ''],[''foo bar'']))'
		description:
			'Is a list of two symbols converted into a list containing those two symbols with a space in between (when the separator is a space)?'.
	self processQuery:
			' if succeeds(listAsSeparatedString(<foo,bar,zork>,['' ''],[''foo bar zork'']))'
		description:
			'Is a list of three symbols converted into a list containing those three symbols with a space between each two symbols (when the separator is a space)?'! !

!DataHandlingTest methodsFor: 'string handling' stamp: 'sp 24/10/11 15:26'!
testmakeFirstCapitalLowerCase
	"self run: #testmakeFirstCapitalLowerCase"

	"makeFirstCapitalLowerCase/2"

	
	self processQuery:
			' if succeeds(makeFirstCapitalLowerCase([''Test''],[''test'']))'
		description: 'Transform first capital into lowercase'.
	self processQuery:
			' if succeeds(makeFirstCapitalLowerCase([''test''],[''test'']))'
		description: 'Does it work if first is already lowercase?'.
	self processQuery:
			' if succeeds(and(makeFirstCapitalLowerCase([''Test''],?x),equals(?x,[''test''])))'
		description: 'Does it work when second argument is variable?'! !

!DataHandlingTest methodsFor: 'list handling' stamp: 'sp 24/10/11 15:26'!
testmember
	"member/2"

	
	self processQuery: ' if succeeds(member(a,<a,b,c>))' description:
			'Does the list <a,b,c> contain a?'.
	self processQuery: ' if succeeds(member(b,<a,b,c>))' description:
			'Does the list <a,b,c> contain b?'.
	self processQuery: ' if fails(member(d,<a,b,c>))' description:
			'Does the list <a,b,c> not contain d?'.
	self processQuery: ' if succeeds(member(p(b),<p(a),q,3,p(b),p(b)>))'
		description: 'Does member/2 work for compound terms?'.
	self
		processQuery: ' if member(?X,<a,b,c>)'
		resultsCheck: [:results | results size = 3]
		description: 'Does the list <a,b,c> contain 3 elements?'.
	self
		processQuery: ' if member(?X,<a,?b,c>)'
		resultsCheck: [:results | results size = 3]
		description:
			'Does member work correctly with lists containing variables?'.
	self processQuery: ' if succeeds(member(a,?L))' description:
			'Does member/2 work when the list is variable?'.
	self processQuery: ' if succeeds(and(member(a,?L),equals(?L,<a|?Rest>)))'
		description:
			'When member/2 is called with a variable list, does it generate a list with the element in it?'.
	self
		processQuery: ' if member(p(?X),<p(a),p(b),p(c)>)'
		resultsCheck: [:results | results size = 3]
		description: 'Does the list <p(a),p(b),p(c)> contain 3 elements?'.
	self processQuery: ' if fails(member(?X,?L))' description:
			'Does the predicate fail when called with two variables?'! !

!DataHandlingTest methodsFor: 'list handling' stamp: 'sp 24/10/11 15:26'!
testnoDups
	"self run: #testnoDups"

	"self debug: #testnoDups"

	
	self processQuery:
			' if succeeds(and(noDups(<a,b,d,c>,?nodups),sameElements(?nodups,<a,b,d,c>)))'
		description:
			'What is the non-duplicates list of a non-duplicates containing list?'.
	self processQuery: ' if succeeds(noDups(<a,b,d,c>,<a,b,d,c>))'
		description:
			'Test the non-duplicates list of a non-duplicates containing list.'.
	self processQuery:
			' if succeeds(and(noDups(<a,b,d,b,a,c,b,c>,?nodups),length(?nodups,4)))'
		description: 'Does noDups/2 actually remove duplicates?'.
	self processQuery: ' if succeeds(noDups(<a,b,d,b,a,c,b,c>,<a,b,d,c>))'
		description:
			'Test the non-duplicates list of a list containing duplicates.'.
	self processQuery: ' if succeeds(noDups(<a,b,d,b,a,c,b,c>,<d,c,b,a>))'
		description:
			'Test the non-duplicates list of a list containing duplicates.'.
	self processQuery:
			' if succeeds(and(noDups(<a,b,d,b,a,c,c,c,b,a>,?nodups),sameElements(?nodups,<a,b,d,c>)))'
		description:
			'What is the non-duplicates list of a duplicates containing list?'.
	self processQuery: ' if succeeds(noDups(<a,b,d,c>,<c,b,a,d>))'
		description:
			'What is the list with duplicates for a result list containing no duplicates?'.
	self processQuery: ' if fails(noDups(?res,<c,b,a,d,c>))' description:
			'Does noDups/2 fail when first argument is uninstantiated?'.
	self processQuery: ' if succeeds(noDups(<[Array]>,?no))' description:
			'Does noDups/2 succeeds for <[Array]>?'! !

!DataHandlingTest methodsFor: 'list handling' stamp: 'sp 24/10/11 15:26'!
testnonEmptyList
	"nonEmptyList/1"

	
	self processQuery: ' if fails(nonEmptyList(<>))' description:
			'Is the empty list <> not non-empty?'.
	self processQuery: ' if succeeds(nonEmptyList(<a,b,c,b>))' description:
			'Is <a,b,c> a non-empty list?'! !

!DataHandlingTest methodsFor: 'term type checking' stamp: 'sp 24/10/11 15:26'!
testnonground
	"nonground/1"

	
	self processQuery: ' if succeeds(nonground(?X))' description:
			'Is a variable not ground?'.
	self processQuery: ' if fails(nonground(<>))' description:
			'Is an empty list ground?'.
	self processQuery: ' if succeeds(nonground(<a,b,?x,d>))' description:
			'Is a list that contains a variable nonground?'.
	self processQuery: ' if fails(nonground(<a,b,c,d>))' description:
			'Is a fully instantiated list ground?'! !

!DataHandlingTest methodsFor: 'term type checking' stamp: 'AndyKellens 7/30/2012 16:47'!
testnonvar
	"nonvar/1"

	
	self processQuery: ' if fails(nonvar(?X))' description:
			'Does nonvar/1 fail for a variable?'.
	self processQuery: ' if succeeds(nonvar(<>))' description:
			'Does nonvar/1 succeed for a non variable such as an empty list?'.
	self processQuery: ' if succeeds(nonvar(<a,b,?x,d>))' description:
			'Does nonvar/1 succeed for a list (even when that list may contain variables)?'.
	self processQuery: ' if succeeds(nonvar(<a,b,c,d>))' description:
			'Does nonvar/1 succeed for a fully instantiated list?'.
	self processQuery: ' if succeeds(nonvar(hallo))' description:
			'Does nonvar/1 succeed for a symbol?'.
	self processQuery: ' if succeeds(nonvar(5))' description:
			'Does nonvar/1 succeed for a number?'.
	self processQuery: ' if succeeds(nonvar([TestClass1]))' description:
			'Does nonvar/1 succeed for a Smalltalk class?'.
	self processQuery: ' if succeeds(nonvar([NonExistingClass]))' description:
			'Does nonvar/1 succeed for a non-existing class? (Even if the class does not exist, it is NOT a logic variable, hence nonvar/1 must succeed.)'! !

!DataHandlingTest methodsFor: 'term type checking' stamp: 'sp 24/10/11 15:26'!
testquotedCode
	"self run: #testquotedCode"

	
	self processQuery: ' if quotedCode({a b c})' description:
			'Detect quotedcode as quotedcode?'.
	self processQuery: ' if not(quotedCode(foo))' description:
			'Fail if not quotedcode?'.
	self processQuery: ' if not(quotedCode(?x))' description:
			'Fail if not quotedcode?'! !

!DataHandlingTest methodsFor: 'list handling' stamp: 'sp 24/10/11 15:26'!
testremove
	"remove/3"

	
	self processQuery:
			' if succeeds(remove(<[1],[2],[3],[4],[5]>,[4],<[1],[2],[3],[5]>))'
		description: 'Remove an element from a list'.
	self processQuery: ' if succeeds(remove(<1,2,3,4,5>,4,?L))' description:
			'Remove an element from a list'.
	self processQuery:
			' if succeeds(and(remove(<1,2,3,4,5>,4,?L),equals(?L,<1,2,3,5>)))'
		description: 'Remove an element from a list'.
	self
		processQuery: ' if remove(<1,2,3,2,1>,1,?L)'
		resultsCheck: [:results | results size = 1]
		description: 'Only first occurence is removed'.
	self processQuery: ' if fails(remove(<1,2,3,2,1>,1,<1,2,3,2>))'
		description: 'Only first occurence is removed'.
	self processQuery: ' if succeeds(remove(<1,2,3,2,1>,?X,?L))' description:
			'Remove some element from some list.'.
	self processQuery: ' if succeeds(remove(?L,4,<a,b,c>))' description:
			'Remove can also be called with first argument variable.'.
	self
		processQuery: ' if remove(?L,4,<a,b,c>)'
		resultsCheck: [:results | results size = 4]
		description:
			'There are 4 ways to re-insert an element in a list of size 3'! !

!DataHandlingTest methodsFor: 'the clause database' stamp: 'sp 24/10/11 15:26'!
testretract1
	"self run:#testretract1"

	
	self repository assert: 'testretract(a).testretract(b)'.
	self processQuery:
			' if retract(testretract(f)),exactlyOne(testretract(b))' description:
			'Is the first clause retracted correctly?'.
	self processQuery: ' if retract(testretract(b)),fails(testretract(b))'
		description: 'Is the second clause retracted correctly?'.
	self processQuery: ' if fails(retract(?foo))' description:
			'We cannot retract unground terms'! !

!DataHandlingTest methodsFor: 'the clause database' stamp: 'sp 24/10/11 15:26'!
testretractall1
	"self run:#testretractall1"

	
	self repository assert: 'testretract2(a).testretract2(a)'.
	self processQuery:
			' if retractall(testretract2(a)),fails(testretract2(a))' description:
			'Are all clauses retracted correctly?'.
	self processQuery: ' if fails(retractall(?foo))' description:
			'We cannot retract unground terms'! !

!DataHandlingTest methodsFor: 'list handling' stamp: 'sp 24/10/11 15:26'!
testreverse
	"reverse/2"

	
	self processQuery:
			' if succeeds(reverse(<[4],[3],[2],[1]>,<[1],[2],[3],[4]>))'
		description:
			'Does reverse work correctly with two instantiated arguments?'.
	self processQuery: ' if succeeds(reverse(?L,<1,2,3,4>))' description:
			'Does reverse work with first argument unbound?'.
	self processQuery:
			' if succeeds(and(reverse(?L,<1,2,3,4>),equals(?L,<4,3,2,1>)))'
		description: 'Does reverse work correctly with first argument unbound?'.
	self processQuery: ' if succeeds(reverse(<1,2,3,4>,?L))' description:
			'Does reverse work with second argument unbound?'.
	self processQuery:
			' if succeeds(and(reverse(<1,2,3,4>,?L),equals(?L,<4,3,2,1>)))'
		description: 'Does reverse work correctly with second argument unbound?'! !

!DataHandlingTest methodsFor: 'list handling' stamp: 'sp 24/10/11 15:26'!
testsameElements
	"sameElements/2"

	
	self processQuery: ' if succeeds(sameElements(<>,<>))' description:
			'Has the empty list the same elements as the empty list?'.
	self processQuery: ' if succeeds(sameElements(<a,b,c,d>,<d,c,a,b>))'
		description: 'Are two lists of symbols with permuted elements the same?'.
	self processQuery: ' if sameElements(<1,2,3>,<2,1,3>)' description:
			'Are two lists of numbers with permuted elements the same?'.
	self processQuery:
			' if succeeds(sameElements(<[1],[2],[3]>,<[2],[1],[3]>))' description:
			'Are two lists of numbers with permuted elements the same?'.
	self processQuery: ' if fails(sameElements(<a,b,c>,<d,a,b>))' description:
			'Are two lists with different elements unequal?'! !

!DataHandlingTest methodsFor: 'string handling' stamp: 'sp 24/10/11 15:26'!
teststartsAndEndsWith
	"self run:#teststartsAndEndsWith"

	"startsAndEndsWith/3"

	
	self processQuery:
			' if succeeds(startsAndEndsWith([''aTest''],[''a''],[''Test'']))'
		description: 'Does a certain string start and end with given substrings?'.
	self processQuery:
			' if succeeds(startsAndEndsWith([''aTest''],[''a''],?End))' description:
			'Does a certain string start with a certain prefix?'.
	self processQuery:
			' if succeeds(and(startsAndEndsWith([''aTest''],[''a''],?End),equals(?End,[''Test''])))'
		description:
			'Does a certain string start with a certain prefix and is the
		remainder of the string computed correctly?'.
	self processQuery: ' if succeeds(startsAndEndsWith(aTest,aTes,?E))'
		description: 'Does a certain string start with a certain prefix?'.
	self processQuery: ' if succeeds(startsAndEndsWith(aTest,aTest,[#'''']))'
		description: 'Does a certain string start with a certain prefix?'.
	self processQuery: ' if succeeds(startsAndEndsWith(aTest,[#''''],aTest))'
		description: 'Does a certain string start with a certain prefix?'.
	self processQuery: ' if succeeds(startsAndEndsWith(aTest,ate,?E))'
		description:
			'Is the startsAndEndsWith predicate case INsensitive on second argument?'.
	self processQuery: ' if fails(startsAndEndsWith(aTest,aTe,sT))'
		description:
			'Is the startsAndEndsWith predicate case SENSITIVE on third argument?'! !

!DataHandlingTest methodsFor: 'string handling' stamp: 'sp 24/10/11 15:26'!
teststartsWith
	"test run:#teststartsWith"

	"startsWith/2"

	
	self processQuery: ' if succeeds(startsWith([''aTest''],[''a'']))'
		description: 'Does a certain string start with a certain prefix?'.
	self processQuery: ' if succeeds(startsWith([''aTest''],aT))' description:
			'Does a certain string start with a certain prefix?'.
	self processQuery: ' if succeeds(startsWith(aTest,[''aTe'']))'
		description: 'Does a certain string start with a certain prefix?'.
	self processQuery: ' if succeeds(startsWith(aTest,aTes))' description:
			'Does a certain string start with a certain prefix?'.
	self processQuery: ' if succeeds(startsWith(aTest,aTest))' description:
			'Does a certain string start with a certain prefix?'.
	self processQuery: ' if succeeds(startsWith(aTest,['''']))' description:
			'Does a certain string start with a certain prefix?'.
	self processQuery: ' if succeeds(startsWith(aTest,atest))' description:
			'Is the startsWith predicate case INsensitive?'.
	self processQuery: ' if succeeds(startsWith([''aTest''],[''atest'']))'
		description: 'Is the startsWith predicate case INsensitive?'! !

!DataHandlingTest methodsFor: 'term type checking' stamp: 'sp 24/10/11 15:26'!
teststring
	"self run: #teststring"

	"string/1"

	
	self processQuery: ' if fails(string(?X))' description:
			'Is a variable not a string?'.
	self processQuery: ' if fails(string(1))' description:
			'Is a number not a string?'.
	self processQuery: ' if fails(string([Object]))' description:
			'Is a Smalltalk class not a string?'.
	self processQuery: ' if succeeds(string(hallo))' description:
			'Is a symbol a string?'.
	self processQuery: ' if succeeds(string([#hallo]))' description:
			'Is a Smalltalk symbol a string?'.
	self processQuery: ' if succeeds(string([''hello there'']))' description:
			'Is a Smalltalk string a string?'! !

!DataHandlingTest methodsFor: 'pattern matching' stamp: 'sp 24/10/11 15:26'!
teststringOrAtom
	"self run:#teststringOrAtom"

	
	self processQuery: ' if succeeds(stringOrAtom([''hallo'']))' description:
			'stringOrAtom succeeds when a string is given as input'.
	self processQuery: ' if succeeds(stringOrAtom(hallo))' description:
			'stringOrAtom succeeds when an atom is given as input'! !

!DataHandlingTest methodsFor: 'string handling' stamp: 'sp 24/10/11 15:26'!
teststringOrAtomSplit
	"self run:#teststringOrAtomSplit"

	"stringOrAtomSplit/4"

	
	self processQuery:
			' if succeeds(stringOrAtomSplit(TesterDeTester,est,T,erDeTester))'
		description:
			'Does stringOrAtomSplit/4 work correctly with 4 instantiated arguments?'.
	self processQuery:
			' if succeeds(stringOrAtomSplit(TesterDeTester,est,TesterDeT,er))'
		description:
			'Does stringOrAtomSplit/4 work correctly with 4 instantiated arguments?'.
	self processQuery:
			' if succeeds(stringOrAtomSplit(TesterDeTester,est,?Before,?After))'
		description:
			'Does stringOrAtomSplit/4 work with last two arguments uninstantiated?'.
	self
		processQuery: ' if stringOrAtomSplit(TesterDeTester,est,?Before,?After)'
		resultsCheck: [:results | results size = 2]
		description:
			'Does stringOrAtomSplit/4 produce more than one result if necessary?'.
	self processQuery:
			' if succeeds(stringOrAtomSplit([''TesterDeTester''],[''est''],[''T''],[''erDeTester'']))'
		description:
			'Does stringOrAtomSplit/4 work correctly with 4 instantiated arguments?'.
	self processQuery:
			' if succeeds(stringOrAtomSplit([''TesterDeTester''],[''est''],[''TesterDeT''],[''er'']))'
		description:
			'Does stringOrAtomSplit/4 work correctly with 4 instantiated arguments?'.
	self processQuery:
			' if succeeds(stringOrAtomSplit([''TesterDeTester''],[''est''],?Before,?After))'
		description:
			'Does stringOrAtomSplit/4 work with last two arguments uninstantiated?'.
	self
		processQuery:
			' if stringOrAtomSplit([''TesterDeTester''],[''est''],?Before,?After)'
		resultsCheck: [:results | results size = 2]
		description:
			'Does stringOrAtomSplit/4 produce more than one result if necessary?'! !

!DataHandlingTest methodsFor: 'string handling' stamp: 'sp 24/10/11 15:26'!
teststringOrAtomStartsWith
	"self run:#teststringOrAtomStartsWith"

	"stringOrAtomStartsWith/2"

	
	self processQuery:
			' if succeeds(stringOrAtomStartsWith([''aTest''],[''a''],[''Test'']))'
		description: 'Does a certain string start and end with given substrings?'.
	self processQuery:
			' if succeeds(stringOrAtomStartsWith([''aTest''],[''a''],?End))'
		description: 'Does a certain string start with a certain prefix?'.
	self processQuery:
			' if succeeds(and(stringOrAtomStartsWith([''aTest''],[''a''],?End),equals(?End,[''Test''])))'
		description:
			'Does a certain string start with a certain prefix and is the
		remainder of the string computed correctly?'.
	self processQuery: ' if succeeds(stringOrAtomStartsWith(aTest,aTes,?E))'
		description: 'Does a certain atom start with a certain atom as prefix?'.
	self processQuery:
			' if succeeds(stringOrAtomStartsWith(aTest,aTest,[#'''']))' description:
			'Does a certain atom start with a certain atom as prefix?'.
	self processQuery:
			' if succeeds(stringOrAtomStartsWith(aTest,[#''''],aTest))' description:
			'Does a certain string or atom start with a certain prefix?'.
	self processQuery: ' if succeeds(stringOrAtomStartsWith(aTest,ate,?E))'
		description:
			'Is the stringOrAtomStartsWith predicate case INsensitive on second argument?'.
	self processQuery: ' if fails(stringOrAtomStartsWith(aTest,aTe,sT))'
		description:
			'Is the stringOrAtomStartsWith predicate case SENSITIVE on third argument?'! !

!DataHandlingTest methodsFor: 'pattern matching' stamp: 'sp 24/10/11 15:26'!
teststringOrAtomToChars
	"self run:#teststringOrAtomToChars"

	"This default test, which always fails, was generated automatically.
	 Please replace with an adequate test of predicate stringOrAtomToChars!!"

	
	self processQuery:
			' if succeeds(stringOrAtomToChars([''hello''],<[#h],[#e],[#l],[#l],[#o]>))'
		description: 'to be tested'	"MORE TESTS NEED TO BE ADDED HERE!!!!!!"! !

!DataHandlingTest methodsFor: 'string handling' stamp: 'sp 24/10/11 15:26'!
teststringSplit
	"self run:#teststringSplit"

	"stringSplit/4"

	
	self processQuery:
			' if succeeds(stringSplit([''TesterDeTester''],[''est''],[''T''],[''erDeTester'']))'
		description:
			'Does stringSplit/4 work correctly with 4 instantiated arguments?'.
	self processQuery:
			' if succeeds(stringSplit([''TesterDeTester''],[''est''],[''TesterDeT''],[''er'']))'
		description:
			'Does stringSplit/4 work correctly with 4 instantiated arguments?'.
	self processQuery:
			' if succeeds(stringSplit([''TesterDeTester''],[''est''],?Before,?After))'
		description:
			'Does stringSplit/4 work with last two arguments uninstantiated?'.
	self
		processQuery:
			' if stringSplit([''TesterDeTester''],[''est''],?Before,?After)'
		resultsCheck: [:results | results size = 2]
		description:
			'Does stringSplit/4 produce more than one result if necessary?'! !

!DataHandlingTest methodsFor: 'pattern matching' stamp: 'sp 24/10/11 15:26'!
teststringToChars
	"self run:#teststringToChars"

	
	self processQuery:
			' if succeeds(stringToChars([''hello''],<[#h],[#e],[#l],[#l],[#o]>))'
		description: 'to be tested'	"MORE TESTS NEED TO BE ADDED HERE!!!!!!"! !

!DataHandlingTest methodsFor: 'set handling checks' stamp: 'sp 24/10/11 15:26'!
testsubSet
	"subSet/2"

	
	self processQuery: ' if succeeds(subSetOf(<a,b,c>,<d,a,c,b,f>))'
		description: 'Is a certain set a subset of another one?'.
	self processQuery: ' if fails(subSetOf(<a,b,q,c>,<d,a,c,b,f>))'
		description: 'Is a certain set no subset of another one?'.
	self processQuery: ' if succeeds(subSetOf(<>,<d,a,c,b,f>))' description:
			'Is the empty set a subset of another set?'.
	self processQuery: ' if succeeds(subSetOf(<>,<>))' description:
			'Is the empty set a subset of itself?'! !

!DataHandlingTest methodsFor: 'term type checking' stamp: 'sp 24/10/11 15:26'!
testsymbol
	"self run: #testsymbol"

	"symbol/1"

	
	self processQuery: ' if fails(symbol(?X))' description:
			'Is a variable not a symbol?'.
	self processQuery: ' if fails(symbol(1))' description:
			'Is a number not a symbol?'.
	self processQuery: ' if fails(symbol([Object]))' description:
			'Is a Smalltalk class not a symbol?'.
	self processQuery: ' if succeeds(symbol(hallo))' description:
			'Check shortcut notation for symbol'.
	self processQuery: ' if succeeds(symbol([#hallo]))' description:
			'Is #hallo a symbol ?'.
	self processQuery: ' if fails(symbol([''hello there'']))' description:
			'Is a Smalltalk string not a symbol?'! !

!DataHandlingTest methodsFor: 'list handling' stamp: 'sp 24/10/11 15:26'!
testtail
	"tail/2"

	
	self processQuery: ' if succeeds(tail(<b,c>,<a,b,c>))' description:
			'Does tail/2 work correctly?'.
	self processQuery: ' if succeeds(tail(<c>,<b,c>))' description:
			'Does tail/2 work correctly?'.
	self processQuery: ' if succeeds(tail(<>,<c>))' description:
			'Does tail/2 work correctly?'.
	self processQuery: ' if succeeds(tail(?x,<a,b,c>))' description:
			'Does tail/2 work when first argument is unbound?'.
	self processQuery: ' if succeeds(and(tail(?x,<a,b,c>),equals(?x,<b,c>)))'
		description: 'Does tail/2 work correctly when first argument is unbound?'.
	self processQuery: ' if succeeds(tail(?x,<a|?y>))' description:
			'Does tail/2 work when rest of list is variable?'.
	self processQuery: ' if succeeds(and(tail(?x,<a|?y>),equals(?x,?y)))'
		description: 'Does tail/2 work correctly when rest of list is variable?'.
	self processQuery: ' if succeeds(tail(?x,<a|?x>))' description:
			'Does tail/2 work correctly when rest of list is variable?'.
	self processQuery: ' if fails(tail(?x,<>))' description:
			'Does the empty list have no tail?'! !

!DataHandlingTest methodsFor: 'set handling operators' stamp: 'sp 24/10/11 15:26'!
testunion
	"union/3"

	
	self processQuery: ' if succeeds(union(<a,e,y,f,u,r,d>,<e,k,f,r,u>,?x))'
		description: 'Is it possible to compute the union of two sets?'.
	self processQuery:
			' if succeeds(union(<a,b,c,d>,<e,f,g,h>,<a,b,c,d,e,f,g,h>))'
		description: 'Is the union of two sets correctly computed?'.
	self processQuery:
			' if succeeds(union(<a,b,c,d>,<b,e,c,a,d>,<a,b,c,d,e>))' description:
			'Is the union of two sets correctly computed?'.
	self processQuery:
			' if succeeds(and(equals(?Set,<a,b,c,d>),union(?Set,<>,?Set)))'
		description: 'Is the union of a set with the empty set the same set?'.
	self processQuery:
			' if succeeds(and(equals(?Set,<a,b,c,d>),union(<>,?Set,?Set)))'
		description: 'Is the union of the empty set with a set the same set?'.
	self processQuery: ' if succeeds(union(<>,<>,<>))' description:
			'Is the union of two empty sets the empty set?'! !

!DataHandlingTest methodsFor: 'term type checking' stamp: 'sp 24/10/11 15:26'!
testvar
	"var/1"

	
	self processQuery: ' if succeeds(var(?X))' description:
			'Does var/1 succeed for a variable?'.
	self processQuery: ' if fails(var(<>))' description:
			'Does var/1 fail for a non variable such as an empty list?'.
	self processQuery: ' if fails(var(<a,b,?x,d>))' description:
			'Does var/1 fail for a list (even when that list may contain variables)?'.
	self processQuery: ' if fails(var(<a,b,c,d>))' description:
			'Does var/1 fail for a fully instantiated list?'.
	self processQuery: ' if fails(var(hallo))' description:
			'Does var/1 fail for a symbol?'.
	self processQuery: ' if fails(var(5))' description:
			'Does var/1 fail for a number?'.
	self processQuery: ' if fails(var([TestClass1]))' description:
			'Does var/1 fail for a Smalltalk class?'.
	self processQuery: ' if fails(var([NonExistingClass]))' description:
			'Does var/1 fail for a non-existing class? (Even if the class does not exist, it is NOT a logic variable, hence var/1 cannot fail!!)'! !

LogicTests subclass: #ErrorHandlingTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOUL-Tests'!

!ErrorHandlingTest methodsFor: 'private' stamp: 'sp 24/10/11 15:26'!
inspectorClassUsedFor: anObject
	
	^anObject inspectorClass! !

!ErrorHandlingTest methodsFor: 'debugging' stamp: 'sp 24/10/11 15:26'!
testconditionalProceed
	"self run:#testconditionalProceed"

	"The conditonalProceed predicate opens a dialog box with the supplied string, and succeeds if the user answers yes or fails if the user answers no.
	 This is not tested in the unit test, because the result needs user interaction, and depends on the choice of the user.
	You can manually test the predicate using the following statements:
		The result of the following should be true if you answer 'Yes' in the dialog:
			(SOULEvaluator eval: 'if conditionalProceed([''Answer YES''])' in: Factory defaultRepository)
				allResults success == true.

		The result of the following should be true if you answer 'No' in the dialog:
			(SOULEvaluator eval: 'if conditionalProceed([''Answer NO''])' in: Factory defaultRepository)
				allResults success == false.
	"

	
	self processQuery: ' if [true]' description:
			'Needs to be tested manually, since user interation is required (see the comment in #testconditionalProceed).'! !

!ErrorHandlingTest methodsFor: 'errors' stamp: 'AndyKellens 11/2/2011 09:53'!
testerror
	"self run:#testerror"

	"Normally, one would do:
		self should: [query allResults] raise: Error
	But I wanted to check the error message as well, so I do this more extensive version."

	
	| query |
	query := SoulFactory current evaluatorForLogicQuery
		eval: 'if error([''Error for the SUnit testcase''])'
		in: self repository.
	[query allResults]
		on: SoulRuntimeException
		do:
			[:exc | self assert: exc messageText = 'SOUL >> Error for the SUnit testcase']! !

!ErrorHandlingTest methodsFor: 'debugging' stamp: 'AndyKellens 11/2/2011 09:53'!
testinspectTerm
	"self run:#testinspectTerm"

	"This test checks whether the inspectTerm predicate indeed opens an inspector on the predicate given (tested with a string). If an inspector is found to exist on this string, it is closed."

	| query aString queryString |
	aString := '$$TestString for SUnit test testInspectTerm$$'.
	queryString := 'if inspectTerm([''' , aString , '''])'.
	query := SoulFactory current evaluatorForLogicQuery eval: queryString in: self repository.
	[ query allResults ]
		on: Exception
		do: [ :exc | 
			| window |
			window := SystemWindow allInstances detect: [ :each | (each model isKindOf: Inspector) and:[each model object = aString]] ifNone: [ nil ].
			window ifNotNil: [ window delete ].
			self assert: exc asString = 'Halt' & window notNil ]! !

LogicTests subclass: #IOTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOUL-Tests'!

!IOTest methodsFor: 'output' stamp: 'sp 24/10/11 15:26'!
testnewline
	"self run: #testnewline"

	"newline/0"

	
	self processQuery: ' if succeeds(newline(?dummy))' description:
			'Does this bring the cursor to a new line on the Transcript?'! !

!IOTest methodsFor: 'printing on Transcript' stamp: 'sp 24/10/11 15:26'!
testprologoutput
	"newline/0"

	"write/1"

	"writeLn/1"

	"This test should print out the following on the Transcript: 
	Language: It`s a virus. 
	-- Laurie Anderson.
	"

	
	self processQuery:
			' if succeeds(and(write([''Language: It`s a virus.'']),newline(?dummy),writeLn([''-- Laurie Anderson.'']),write([''The above should have printed nice!!''])))'
		description: 'Does this print something nice on the Transcript?'! !

!IOTest methodsFor: 'output' stamp: 'sp 24/10/11 15:26'!
testwrite
	"self run: #testwrite"

	"write/1"

	
	self processQuery: ' if succeeds(write(hello))' description:
			'Does this print the string hello on the Transcript?'! !

!IOTest methodsFor: 'output' stamp: 'sp 24/10/11 15:26'!
testwriteLn
	"self run: #testwriteLn"

	"writeLn/1"

	
	self processQuery: ' if succeeds(writeLn(hello))' description:
			'Does this print the string hello on the Transcript and then go to a new line?'! !

!IOTest methodsFor: 'output' stamp: 'sp 24/10/11 15:26'!
testwriteLnLst
	"self run: #testwriteLnLst"

	"writeLnLst/1"

	
	self processQuery: ' if succeeds(writeLnLst(<a,b,c>))' description:
			'Does this write the elements a, b, c on separate lines of the Transcript?'! !

!IOTest methodsFor: 'output' stamp: 'sp 24/10/11 15:26'!
testwriteLst
	"self run: #testwriteLst"

	"writeLst/1"

	
	self processQuery: ' if succeeds(writeLst(<a,b,c>))' description:
			'Does this write the elements a, b, c one after the other on the Transcript?'! !

!IOTest methodsFor: 'output' stamp: 'sp 24/10/11 15:26'!
testwriteLstLn
	"self run: #testwriteLstLn"

	"writeLstLn/1"

	
	self processQuery: ' if succeeds(writeLstLn(<a,b,c>))' description:
			'Does this write the elements a, b, c one after the other on the Transcript and then go to a new line?'! !

LogicTests subclass: #KernelTest
	instanceVariableNames: ''
	classVariableNames: 'TestRepository'
	poolDictionaries: ''
	category: 'SOUL-Tests'!

!KernelTest class methodsFor: 'testRepository' stamp: 'sp 24/10/11 15:26'!
testRepository
	
	^TestRepository! !

!KernelTest class methodsFor: 'testRepository' stamp: 'sp 24/10/11 15:26'!
testRepository: aRep
	
	TestRepository := aRep! !

!KernelTest methodsFor: 'private' stamp: 'AndyKellens 11/2/2011 09:53'!
assertExpressionPrintsAndParsesSanely: soulExpressionString
	
	| representation |
	representation := SoulFactory current makeParser parse: soulExpressionString.
	self assertRepresentationPrintsAndParsesSanely: representation! !

!KernelTest methodsFor: 'private' stamp: 'AndyKellens 11/2/2011 09:53'!
assertRepresentationPrintsAndParsesSanely: representation
	
	| printedExpressionString reparsedRepresentation reprintedExpressionString |
	printedExpressionString := representation printString.
	reparsedRepresentation := SoulFactory current makeParser parse:
			printedExpressionString.
	reprintedExpressionString := reparsedRepresentation printString.
	self assert: printedExpressionString = reprintedExpressionString! !

!KernelTest methodsFor: 'parser-printer' stamp: 'AndyKellens 11/2/2011 09:53'!
should: expression1 printAs: expression2
	
	| representation |
	representation := SoulFactory current makeParser parse: expression1.
	self assert: representation printString = expression2! !

!KernelTest methodsFor: 'prolog translation' stamp: 'sp 24/10/11 15:26'!
should: soulString translateAs: prologString
	
	| translated parsed |
	parsed := SoulParser parse: soulString.
	translated := parsed terms first asProlog.
	self
		assert: translated = (prologString , '.' , (String with: Character cr))! !

!KernelTest methodsFor: 'evaluator' stamp: 'sp 24/10/11 15:26'!
testBasicRecursion
	"self run:#testBasicRecursion"

	
	self repository assert: 'plus(pair(d,?x),?y,pair(d,?z)) if plus(?x,?y,?z)'.
	self repository assert: 'plus(zero,?x,?x)'.
	self
		processQuery: ' if plus(?x,?y,pair(d,pair(d,pair(d,zero))))'
		resultsCheck: [:results | results size = 4]
		description: 'Basic recursion test'
		repository: self repository.
	self
		processQuery:
			' if plus(pair(d,pair(d,pair(d,zero))),zero,pair(d,pair(d,pair(d,zero))))'
		description: 'Basic recursion test'
		repository: self repository.
	self
		processQuery:
			' if plus(pair(d,zero),pair(d,pair(d,zero)),pair(d,pair(d,pair(d,zero))))'
		description: 'Basic recursion test'
		repository: self repository.
	self
		processQuery:
			' if plus(pair(d,pair(d,zero)),pair(d,zero),pair(d,pair(d,pair(d,zero))))'
		description: 'Basic recursion test'
		repository: self repository.
	self repository retractAll: 'plus/3'! !

!KernelTest methodsFor: 'symbiosis' stamp: 'sp 24/10/11 15:26'!
testConstants
	"self run:#testConstants"

	
	self processQuery: ' if succeeds(equals(a,[#a]))' description:
			'Testing shortcut notation for constant symbols.'.
	self processQuery: ' if succeeds(equals([#a],a))' description:
			'Testing shortcut notation for constant symbols.'.
	self processQuery: ' if succeeds(equals(1,[1]))' description:
			'Testing shortcut notation for constant integers.'.
	self processQuery: ' if succeeds(equals([1],1))' description:
			'Testing shortcut notation for constant integers.'.
	self processQuery: ' if fails(equals([Array],Array))' description:
			'Of course there is no shortcut notation for Smalltalk classes."'! !

!KernelTest methodsFor: 'parser-printer' stamp: 'sp 24/10/11 15:26'!
testConversion
	
	self
		assert:
			(SoulParseTreeBuilder new buildConstant: #bar) smalltalkBlockString = 'bar'.
	self
		assert:
			(SoulParseTreeBuilder new buildConstant: 'bar') smalltalkBlockString =
					'bar'.
	self
		assert:
			(SoulParseTreeBuilder new buildConstant: '#bar') smalltalkBlockString =
					'#bar'.
	self
		assert:
			(SoulParseTreeBuilder new buildConstant: #'instance creation')
				smalltalkBlockString = 'instance creation'! !

!KernelTest methodsFor: 'evaluator' stamp: 'sp 24/10/11 15:26'!
testCut
	"self run:#testCut"

	
	self repository assert:
			'foo(?x) if equals(a,?x),!!.foo(?x) if equals(?x,b)'.
	self
		processQuery: ' if succeeds(foo(a))'
		description: 'Does the cut work?'
		repository: self repository.
	self
		processQuery: ' if foo(?x)'
		resultsCheck: [:results | results size = 1]
		description: 'Does the cut work?'
		repository: self repository.
	self
		processQuery: ' if succeeds(foo(b))'
		description: 'Does the cut work?'
		repository: self repository.
	self
		processQuery: ' if member(?x,<1,2,3>),testcutonfinal(?c)'
		resultsCheck: [:results | results size = 9]
		description: 'Does the cut work when the last clause contains the cut?'.
	self repository retractAll: 'foo/1'! !

!KernelTest methodsFor: 'evaluator' stamp: 'sp 24/10/11 15:26'!
testDynamicPredicate
	"Test the dynamic predicate for failure"

	
	self
		processQuery: ' if fails(testDynamicPredicate)'
		description: 'Dynamic Predicate test'
		repository: self repository.	"Test a non-dynamic predicate for error"

	[self
		processQuery: ' if notTestDynamicPredicate'
		description: 'Dynamic Predicate test'
		repository: self repository]
		on: Error
		do: [:ex | true]! !

!KernelTest methodsFor: 'evaluator' stamp: 'sp 24/10/11 15:26'!
testFactsInMultipleClasses
	"self run:#testFactsInMultipleClasses"

	
	self
		processQuery: ' if fact3(?X)'
		resultsCheck: [:results | results size = 4]
		description:
			'Are all definitions of a predicate found when its definition is spread over multiple classes?'! !

!KernelTest methodsFor: 'evaluator' stamp: 'sp 24/10/11 15:26'!
testList
	"self run:#testList"

	
	self processQuery: ' if equals(<a,b,c>,<a,b,c>)' description:
			'Is the shorthand notation of list correct?'.
	self processQuery: ' if fails(equals(<a|b>,<a,b>))' description:
			'Is the cons-cel construction correct?'! !

!KernelTest methodsFor: 'evaluator' stamp: 'sp 24/10/11 15:26'!
testListCollectionUnification
	"self run:#testListCollectionUnification"

	
	self processQuery:
			' if equals(<a,b,c>,[OrderedCollection with: #a with: #b with: #c])'
		description: 'Do lists correctly unify with sequenceable collections?'.
	self processQuery:
			' if equals([OrderedCollection with: #a with: #b with: #c],<a,b,c>)'
		description: 'Do lists correctly unify with sequenceable collections?'.
	self processQuery: ' if equals(<>,[OrderedCollection new])' description:
			'Do empty lists correctly unify with empty sequenceable collections?'.
	self processQuery: ' if equals([OrderedCollection new],<>)' description:
			'Do empty lists correctly unify with empty sequenceable collections?'.
	self processQuery:
			' if equals([OrderedCollection
	with: #a
	with: (OrderedCollection with: #b)
	with: #c],<a,<b>,c>)' description: 'Do nested list unify?'.
	self processQuery:
			' if fails(equals([OrderedCollection with: #a with: #b],<a,b,c>))'
		description: 'Does unification fail when elements do not correspond?'.
	self processQuery:
			' if fails(equals([OrderedCollection with: #foo with: #bar],<a,b>))'
		description: 'Does unification fail when elements do not correspond?'! !

!KernelTest methodsFor: 'evaluator' stamp: 'sp 24/10/11 15:26'!
testNegativeVariables
	
	self processQuery: ' if testNegVariable(?a)' description:
			'The most simple case for negative variables'.
	self processQuery: ' if not(testNegVariable(a))' description:
			'The most simple case for negative variables'.
	self processQuery: ' if equals(?x,?a),testNegVariable(?x)' description:
			'The second simplest case for negative variables'.
	self processQuery: ' if equals(?x,a),not(testNegVariable(?x))'
		description: 'The second simplest case for negative variables'.
	self processQuery: ' if testNegVariable(2,?y),equals(?y,2)' description:
			'Do negative variables still work when used in a rule'! !

!KernelTest methodsFor: 'evaluator' stamp: 'sp 24/10/11 15:26'!
testNestedFindall
	
	self
		processQuery:
			' if findall(?xs,findall(?x,fooTest(?x),?xs),?ys),equals(?ys,<<1,2>>)'
		description: 'Nested findall test'
		repository: self repository! !

!KernelTest methodsFor: 'evaluator' stamp: 'sp 24/10/11 15:26'!
testPositiveVariables
	
	self repository assert: 'testPosVariable(+?x)'.
	self processQuery: ' if testPosVariable(a)' description:
			'The most simple case for positive variables'.
	self processQuery: ' if not(testPosVariable(?a))' description:
			'The most simple case for positive variables'.
	self processQuery: ' if equals(?x,a),testPosVariable(?x)' description:
			'The second simplest case for positive variables'.
	self processQuery:
			' if equals(?x,?y),not(testPosVariable(?x)),not(testPosVariable(?y))'
		description: 'The second simplest case for positive variables'.
	self processQuery: ' if equals(?x,a),equals(?x,?y),testPosVariable(?y)'
		description: 'The third simplest case for positive variables'.
	self repository retractAll: 'testPosVariable/1'.
	self repository assert: 'testing(2)'.
	self repository assert: 'testPosVariable(+?x) if testing(?x)'.
	self processQuery: ' if testPosVariable(2)' description:
			'Do positive variables still work when used in a rule'.
	self repository retractAll: 'testPosVariable/1'.
	self repository retractAll: 'testing/1'! !

!KernelTest methodsFor: 'parser-printer' stamp: 'AndyKellens 7/30/2012 16:19'!
testPrint
	"This is a more strict test than the one performed by the sanity test testPrintAndParse. Unlike the sanity test, it really tests that the printer and parser are correct for these expressions and not just sane, but this test is sensitive to small changes in the printer (such as adding extra irrelevant spacing)"

	
	self should: '1' printAs: '1'.
	self should: 'abc' printAs: 'abc'.
	self should: '<1>' printAs: '<1>'.
	self should: '<?x>' printAs: '<?x>'.
	self should: '<1,2>' printAs: '<1,2>'.
	self should: '<1, 2>' printAs: '<1,2>'.
	self should: '< [1], [2] >' printAs: '<[1],[2]>'.
	self should: '<1,2|3>' printAs: '<1,2|3>'.
	self should: '<1,2|<3>>' printAs: '<1,2,3>'.
	self should: '<?x|?y>' printAs: '<?x|?y>'.
	self should: '<?x,?y>' printAs: '<?x,?y>'.
	self should: '<<?x>|<?y>>' printAs: '<<?x>,?y>'.
	self should: '<<?x>|?y>' printAs: '<<?x>|?y>'.
	self should: 'abc(1,2,3)' printAs: 'abc(1,2,3)'.
	self
		should: 'abc(1,xyz(2,2),3) if defg([Cat],[Dog]), hij([?x + ?y])'
		printAs:
			'abc(1,xyz(2,2),3) if '
				,
					(String
						with: Character cr
						with: Character tab) , 'defg([Cat],[Dog]),hij([?x + ?y])'! !

!KernelTest methodsFor: 'parser-printer' stamp: 'sp 24/10/11 15:26'!
testPrintAndParse
	"This does a sanity test of the printer and parser: the given expression is first parsed, and then printed. Then the printed version is again parsed and printed. The two prints should be exactly the same. Note that this is a necessary but not sufficient condition for the print to be correct, since printing everything to the string 'dummy' would also satisfy this test."

	
	self assertExpressionPrintsAndParsesSanely: 'abc'.
	self assertExpressionPrintsAndParsesSanely: '1'.
	self assertExpressionPrintsAndParsesSanely: '<>'.
	self assertExpressionPrintsAndParsesSanely: 'Abc'.
	self assertExpressionPrintsAndParsesSanely: '?x'.
	self assertExpressionPrintsAndParsesSanely: '[ 1 ]'.
	self assertExpressionPrintsAndParsesSanely: 'a(1,2)'.
	self assertExpressionPrintsAndParsesSanely: '[ 1 + ?x ]'.
	self assertExpressionPrintsAndParsesSanely: '<1,2,3>'.
	self assertExpressionPrintsAndParsesSanely: '<1,2|3>'.
	self assertExpressionPrintsAndParsesSanely: '<1,2,3|<>>'.
	self assertExpressionPrintsAndParsesSanely: '<1,2|<3>>'.
	self assertExpressionPrintsAndParsesSanely: '<?x|?y>'.
	self assertExpressionPrintsAndParsesSanely: '<?x|<?y>>'.
	self assertExpressionPrintsAndParsesSanely: 'a(1) if b(2)'.
	self assertExpressionPrintsAndParsesSanely:
			'a(1,2,3) if b(4,5,6), c(7,8,9)'.
	self assertExpressionPrintsAndParsesSanely:
			'a( 1, 2 ,3  ) if b( 4,5,6 ), c(7 , 8 , 9)'! !

!KernelTest methodsFor: 'parser-printer' stamp: 'sp 24/10/11 15:26'!
testPrintAndParseExtensively
	"This does the same sanity test as 'testPrintAndParse' but for all the clauses in LogicStorage."

	
	LogicStorage allClauses
		do: [:clause | self assertRepresentationPrintsAndParsesSanely: clause]! !

!KernelTest methodsFor: 'prolog translation' stamp: 'sp 24/10/11 15:26'!
testPrologTranslation
	"Check translation of variables"

	
	self should: '?x' translateAs: 'X'.
	self should: '?y' translateAs: 'Y'.
	self should: '?' translateAs: '_'.	"Check translation of simple values"

	self should: '1' translateAs: '1'.
	self should: '#bla' translateAs: 'bla'.
	self should: 'bla' translateAs: 'bla'.
	self should: 'Bla' translateAs: '''Bla'''.	"Check translation of compounds in old syntax"

	self should: 'bla(blie,bloe)' translateAs: 'bla(blie,bloe)'.
	self should: 'bla(?x,bloe(1))' translateAs: 'bla(X,bloe(1))'.	"Check translation of lists"

	self should: '<1,2,3>' translateAs: '[1,2,3]'.	" self should: '<1,2|X>' translateAs: '[1,2|X]'. " 	"FAILS, SHOULD BE FIXED"
	"Check translation of compounds in new syntax"

	self should: 'bla: ?x bloe: ?y' translateAs: 'bla_bloe(X,Y)'.
	self should: '?x bla' translateAs: 'bla(X)'.	"Check translation of rules"

	self should: 'bla(?x) if bloe(?y)' translateAs:
			'bla(X) :- 
	bloe(Y)'! !

!KernelTest methodsFor: 'evaluator' stamp: 'sp 24/10/11 15:26'!
testQuotedCode
	"TODO: (KG) I disabled these tests because they cause parsing errors with the LogicTestsRewriterVisitor"

	"self repository assert: 'quotedCodeTest1({a}).quotedCodeTest2({b}).quotedCodeTest3({})'.
	self
		processQuery: 'if succeeds(quotedCodeTest1({a}))'
		description: 'Does QuotedCode unify?'
		repository: self repository.
	self
		processQuery: 'if quotedCodeTest1(?a),quotedCodeTest2(?b),quotedCodeTest3(?c),succeeds(equals({a b},{?a ?b?c}))'
		description: 'Does QuotedCode substitute correctly?'
		repository: self repository.
	self
		processQuery: 'if equals(?x,bla),succeeds(equals({bla},{?x}))'
		description: 'Do constants in QuotedCode substitute correctly?'
		repository: self repository.
	self
		processQuery: 'if equals(?x,{bla foo}),[?x asString = ''bla foo'']'
		description: 'Are quotedcodes downed correctly?'
		repository: self repository.
	self
		processQuery: 'if equals(?x,{''bla'' is bla}),[?x asString = ('''''''',''bla'','''''''','' is bla'')]'
		description: 'Do quotes in QuotedCode substitute correctly?'
		repository: self repository.
	self
		processQuery: 'if equals(?x,[Object]),succeeds(equals({Object},{?x}))'
		description: 'Do smalltalkvalues in QuotedCode substitute correctly?'
		repository: self repository.
	self
		processQuery: 'if equals(?func(?c),{a ?b c}),length(?func,5)'
		description: 'Do Compoundterms and QuotedCode unify correctly?'
		repository: self repository.
	self
		processQuery: 'if equals(?x(?y),{a ?b c}),equals({a b c},{@?x({b})})'
		description: 'Do Compoundterms substitute correctly in QuotedCode?'
		repository: self repository.
	self repository retractall: 'quotedCodeTest1/1'."
	
	! !

!KernelTest methodsFor: 'repositories' stamp: 'AndyKellens 11/2/2011 09:53'!
testScoping
	
	| repA repB |
	repA := SoulFactory current makeRepository new.
	repA compilationFactory: (SoulFactory current).
	repB := SoulFactory current makeRepository new.
	repB compilationFactory: (SoulFactory current).
	repA assert: 'fact(a)'.
	repA assert: 'equals(?x,?x)'.
	repA assert: '?b->fact(c)'.
	repA assert: '?b->fact(d) if fact(a)'.
	repB assert: 'fact(b)'.
	repA
		bind: (Variable name: 'b')
		with: repB.
	self processQuery: ' if fact(?x),equals(?x,a)' description:
			'Does the repository with bindings work as normal?' repository: repA.
	self processQuery: ' if ?b->fact(?x),equals(?x,b)' description:
			'Does the repository lookup work correctly?' repository: repA.
	self processQuery: ' if ?b->fact(d)' description:
			'Does the repository lookup work correctly?' repository: repA.
	self processQuery: ' if fact(b),fact(c)' description:
			'Does the repository addition work correctly?' repository: repB.
	self class testRepository: repB.
	[self processQuery:
			' if equals(?rep,[KernelTest testRepository]),?rep->fact(b)'
		description:
			'Does the lookup mechanism first consider looking in the local environment?'
		repository: repA] ensure: [self class testRepository: nil]! !

!KernelTest methodsFor: 'parser-printer' stamp: 'sp 24/10/11 15:26'!
testSmalltalkTerms
	
	self processQuery: ' if equals([''Is this ok? yeah!!''],?x)' description:
			'Can Smalltalkterms contain question marks and not recognize them as variables?'! !

!KernelTest methodsFor: 'evaluator' stamp: 'sp 24/10/11 15:26'!
testUnboundLookupLoop
	"self run:#testUnboundLookupLoop"

	
	self processQuery: ' if succeeds(equals(?x,?x))' description:
			'Unbound variable lookup-loop test'! !

!KernelTest methodsFor: 'evaluator' stamp: 'AndyKellens 10/28/2011 15:52'!
testVarArgsCompoundWithMessageFunctors
	
	self
		processQuery: ' if equals(#foo:@(?args), ?x foo: ?y)'
		description:
			'Binary message functors unify with variable argument compound terms (direct functor symbol).'
		repository: self repository.
	self
		processQuery:
			' if ?functor equals: #foo:,equals(?functor@(?args),?x foo: ?y)'
		description:
			'Binary message functors unify with variable argument compound terms (indirect functor symbol).'
		repository: self repository.
	self
		processQuery:
			' if ?functor equals: #foo:bar:,equals(?functor@(?args),?x foo: ?y bar: ?z)'
		description:
			'Non-unary message functors unify with variable argument compound terms (indirect functor symbol).'
		repository: self repository.
	self
		processQuery: ' if equals(isFoo@(?args),?x isFoo)'
		description:
			'Unary message functors unify with variable argument compound terms (direct functor symbol).'
		repository: self repository.
	self
		processQuery:
			' if ?functor equals: isFoo,equals(?functor@(?args),?x isFoo)'
		description:
			'Unary message functors unify with variable argument compound terms (indirect functor symbol).'
		repository: self repository! !

!KernelTest methodsFor: 'evaluator' stamp: 'sp 24/10/11 15:26'!
testVariableFunctor
	
	self repository assert: 'foo(?functor(?functor))'.
	self repository assert: 'bla(a)'.
	self
		processQuery: ' if succeeds(foo(bla(bla)))'
		description: 'Variable functor test'
		repository: self repository.
	self
		processQuery: ' if succeeds(and(equals(?name,bla),?name(a)))'
		description: 'Variable functor test'
		repository: self repository.
	self repository retractAll: 'bla/1'.
	self repository retractAll: 'foo/1'! !

!KernelTest methodsFor: 'evaluator' stamp: 'sp 24/10/11 15:26'!
testVariablesAsConditions
	
	self processQuery: ' if equals(?y,5),equals(?x,equals(?y,5)),?x'
		description:
			'Basic test to verify that variables can be used as conditions.'.
	self processQuery: ' if metaFact(?fact),?fact' description:
			'Test to verify that a variable can be used as condition when bound to a term coming from a fact.'.
	self processQuery: ' if and(equals(?x,equals(?y,6)),?x)' description:
			'Test to verify if using variables as conditions works in meta predicates such as "and" as well.'	"TODO: This following test fails!!"
	"	self processQuery: 'if succeeds(equals(?x,equals(?y,6)),?x)'
		description: 'Test to verify if using variables as conditions works in meta predicates such as succeeds as well.'."! !

!KernelTest methodsFor: 'natives' stamp: 'sp 24/10/11 15:26'!
testclause
	
	self processQuery: ' if clause(equals(?a,?b),?body),equals(?body,<>)'
		description: 'Does the clause predicate give an empty body for a fact?'.
	self processQuery: ' if clause(?head,<>),equals(?head,equals(?z,?z))'
		description: 'Is equals(?z,?z) one of the facts in the repository?'.
	self processQuery:
			' if clause(?head,<not(equals(?a,?b))>),equals(?head,differs(?a,?b))'
		description:
			'Is the rule with the body not(equals(?a, ?b)) the one for the predicate differs?'.
	self processQuery:
			' if clause(equals(?avar,[?unboundVar someMessage]),?body)' description:
			'Does clause work correctly when the head contains a compound term with a smalltalk term in it?'! !

!KernelTest methodsFor: 'natives' stamp: 'sp 24/10/11 15:26'!
testenvLookup
	
	self processQuery:
			' if envLookup(?var,?result),equals([?result term],?var),equals([?result envIndex],1)'
		description:
			'Does envLookup return the termApp object for a variable correctly?'.
	self processQuery:
			' if envLookup(const,?result),equals([?result term name],const),equals([?result envIndex],1)'
		description:
			'Does envLookup return the termApp object for a constant correctly?'! !

!KernelTest methodsFor: 'natives' stamp: 'sp 24/10/11 15:26'!
testgetEnv
	
	self processQuery:
			' if getEnv(?env,?id),[(?env isKindOf: Environment) and: [?id isKindOf: Number]]'
		description: 'Does the getEnv predicate return an environment?'.
	self processQuery: ' if getEnv(?env,?id),[?env == env]' description:
			'Does the getEnv predicate return the correct environment?'! !

!KernelTest methodsFor: 'natives' stamp: 'sp 24/10/11 15:26'!
testgetRepository
	
	self processQuery:
			' if getRepository(?rep),[?rep isKindOf: LogicRepository]' description:
			'Does the getRepository predicate return a repository?'! !

!KernelTest methodsFor: 'evaluator' stamp: 'sp 24/10/11 15:26'!
testnot
	"not/1"

	
	self processQuery: ' if succeeds(not(list([1])))' description:
			'Does not/1 succeed when the enclosed query fails?'.
	self processQuery: ' if fails(not(list(<>)))' description:
			'Does fails/1 fail when the enclosed query succeeds?'.
	self processQuery: ' if succeeds(not(not(list(<>))))' description:
			'Does negation of negation succeed when the enclosed query fails?'.
	self processQuery: ' if fails(not(not(list([1]))))' description:
			'Does negation of negation fail when the enclosed query fails?'! !

LogicTests subclass: #LogicProgramControlTest
	instanceVariableNames: ''
	classVariableNames: 'DoTestMemory'
	poolDictionaries: ''
	category: 'SOUL-Tests'!

!LogicProgramControlTest class methodsFor: 'doTestMemory' stamp: 'sp 24/10/11 15:26'!
doTestMemory
	"This shared variable serves as memory to test side-effects in the do/1 predicate"

	
	^DoTestMemory! !

!LogicProgramControlTest class methodsFor: 'doTestMemory' stamp: 'sp 24/10/11 15:26'!
incrementDoTestMemory
	"This shared variable serves as memory to test side-effects in the do/1 predicate"

	
	DoTestMemory := self doTestMemory + 1! !

!LogicProgramControlTest class methodsFor: 'initialize-release' stamp: 'AndyKellens 11/2/2011 09:53'!
initialize
	"Reset the default repository because a logic test repository needs to be included."

	
	SoulFactory resetDefaultRepository! !

!LogicProgramControlTest class methodsFor: 'doTestMemory' stamp: 'sp 24/10/11 15:26'!
resetDoTestMemory
	"This shared variable serves as memory to test side-effects in the do/1 predicate"

	
	DoTestMemory := 0! !

!LogicProgramControlTest methodsFor: 'meta programming' stamp: 'sp 24/10/11 15:26'!
testapply2
	"self run:#testapply2"

	
	self processQuery: ' if succeeds(apply(fact2,<onlyOccurence>))'
		description:
			'This should work because it is equivalent to calling fact2(onlyOccurence) which is supposed to work'.
	self processQuery: ' if succeeds(apply(fact2,<?x>))' description:
			'This should work because it is equivalent to calling fact2(?x) which is supposed to work'.
	self processQuery:
			' if succeeds(and(apply(fact2,<?x>),equals(?x,onlyOccurence)))'
		description:
			'This should work because it is equivalent to calling fact2(?x) which indeed returns onlyOccurence as a possible value for ?x'! !

!LogicProgramControlTest methodsFor: 'meta programming' stamp: 'sp 24/10/11 15:26'!
testcase
	"self run:#testcase"

	
	self processQuery:
			' if equals(?X,0),case(cond(smaller(?X,1),equals(?Result,firstCase)),cond(smaller(?X,4),equals(?Result,secondCase)),cond([true],equals(?Result,thirdCase))),equals(?Result,firstCase)'
		description: 'Does case succeed for the first case?'.
	self processQuery:
			' if equals(?X,2),case(cond(smaller(?X,1),equals(?Result,firstCase)),cond(smaller(?X,4),equals(?Result,secondCase)),cond([true],equals(?Result,thirdCase))),equals(?Result,secondCase)'
		description: 'Does case succeed for the second case?'.
	self processQuery:
			' if equals(?X,5),case(cond(smaller(?X,1),equals(?Result,firstCase)),cond(smaller(?X,4),equals(?Result,secondCase)),cond([true],equals(?Result,thirdCase))),equals(?Result,thirdCase)'
		description: 'Does case succeed for the last case?'.
	self
		processQuery:
			' if equals(?X,0),[LogicProgramControlTest resetDoTestMemory.
true],case(cond(smaller(?X,1),[LogicProgramControlTest incrementDoTestMemory.
true]),cond(smaller(?X,4),[LogicProgramControlTest incrementDoTestMemory.
true]),cond([true],[LogicProgramControlTest incrementDoTestMemory.
true]))'
		resultsCheck: [:result | self class doTestMemory = 1]
		description: 'Check that the case indeed executes only the first action'! !

!LogicProgramControlTest methodsFor: 'sets of solutions' stamp: 'sp 24/10/11 15:26'!
testconstrain
	"self run:#testconstrain"

	"constrain/2"

	"Can the author of constrain/2 (Roel?) please add some adequate tests for the predicate constrain/2 here?"

	
	self
		processQuery: ' if constrain(?c,[Smalltalk allClasses])'
		resultsCheck: [:results | results size = Smalltalk allClasses size]
		description: 'constrain/2'.
	self processQuery: ' if constrain([Object],[Smalltalk allClasses])'
		description: 'constrain/2'! !

!LogicProgramControlTest methodsFor: 'meta programming' stamp: 'sp 24/10/11 15:26'!
testdo1
	"self run:#testdo1"

	"do/1"

	
	self processQuery: ' if do(writeLn([''Hallo'']))' description:
			'Does do/1 succeed when the action succeeds?'.
	self processQuery: ' if do(and(writeLn([''Hallo'']),[false]))'
		description: 'Does do/1 succeed even when the action fails?'.
	self
		processQuery:
			' if [LogicProgramControlTest resetDoTestMemory.
true],do([LogicProgramControlTest incrementDoTestMemory.
true])'
		resultsCheck: [:result | self class doTestMemory = 1]
		description: 'Does do/1 produce the expected side-effect?'.
	self
		processQuery:
			' if [LogicProgramControlTest resetDoTestMemory.
true],do(and([LogicProgramControlTest incrementDoTestMemory.
true],[false]))'
		resultsCheck: [:result | self class doTestMemory = 1]
		description:
			'Does do/1 produce the expected side-effect even when the action fails?'! !

!LogicProgramControlTest methodsFor: 'sets of solutions' stamp: 'sp 24/10/11 15:26'!
testexactlyOne
	"self run:#testexactlyOne"

	"exactlyOne/1"

	
	self processQuery: ' if fails(exactlyOne(fact1(a)))' description:
			'Doe exactlyOne/1 called on a fact fail if there are 3 occurences of that fact?'.
	self processQuery: ' if fails(exactlyOne(fact1(b)))' description:
			'Doe exactlyOne/1 called on a fact fail if there are 2 occurences of that fact?'.
	self processQuery: ' if succeeds(exactlyOne(fact1(c)))' description:
			'Doe exactlyOne/1 called on a fact succeed if there is only 1 occurence of that fact?'.
	self processQuery: ' if fails(exactlyOne(fact1(d)))' description:
			'Doe exactlyOne/1 called on a fact fail if there are no occurences of that fact?'.
	self processQuery: ' if fails(exactlyOne(fact1(?X)))' description:
			'Doe exactlyOne/1 called on a query fail if there are multiple solutions of that query?'.
	self processQuery: ' if succeeds(exactlyOne(fact2(onlyOccurence)))'
		description:
			'Doe exactlyOne/1 called on a fact succeed if there is only 1 occurence of that fact?'.
	self processQuery: ' if succeeds(exactlyOne(fact2(?X)))' description:
			'Doe exactlyOne/1 called on a query succeed if there is only 1 solution to that query?'.
	self processQuery: ' if fails(exactlyOne(member(?X,<a,b,c,d>)))'
		description:
			'Doe exactlyOne/1 called on a query fail if there are many solutions to that query?'.
	self processQuery: ' if succeeds(exactlyOne(one(member(?X,<a,b,c,d>))))'
		description:
			'Doe exactlyOne/1 called on a query succeeds if there is at least one solution to that query, and the query is wrapped in a one/1 statement?'! !

!LogicProgramControlTest methodsFor: 'meta programming' stamp: 'sp 24/10/11 15:26'!
testexists
	"self run:#testexists"

	
	self processQuery: ' if exists(fact2(?x))' description:
			'Does exists/1 work if there is exactly one solution for the query?'.
	self processQuery: ' if exists(fact1(?x))' description:
			'Does exists/1 work if there are multiple solutions for the query?'.
	self processQuery: ' if not(exists(fact1(z)))' description:
			'Does exists/1 fail if there are no solutions for the query?'.
	self processQuery: ' if exists(fact1(a),fact1(a))' description:
			'Does exists/2 work if there is solution for the expression and the query?'.
	self processQuery: ' if exists(fact1(?x),member(?x,<a>))' description:
			'Does exists/2 work if there is solution for the expression and the query?'.
	self processQuery: ' if not(exists(fact1(?x),member(?x,<z>)))'
		description:
			'Does exists/2 fail if there is solution for the expression and not for the query?'.
	self processQuery: ' if not(exists(fact1(z),member(a,<a>)))' description:
			'Does exists/2 fail if there is no solution for the expression but there is one for the query?'.
	self processQuery: ' if not(exists(fact1(z),member(z,<a>)))' description:
			'Does exists/2 fail if there is no solution for the expression and for the query?'! !

!LogicProgramControlTest methodsFor: 'meta programming' stamp: 'sp 24/10/11 15:26'!
testexists1
	"self run:#testexists1"

	"exists/1"

	
	self processQuery: ' if succeeds(exists(list(<>)))' description:
			'Does there exist an empty list?'.
	self processQuery: ' if fails(exists(list([1])))' description:
			'Does no solution exist to the query list([1])?'.
	self processQuery: ' if exists(member(1,<1,2,3>))' description:
			'Does there exists a member 1 in the list <1,2,3>?'.
	self processQuery: ' if exists(member(1,<2,1,3,1>))' description:
			'Does there exist a member 1 in a list containing multiple occurences of 1? (Note that this query is supposed to return one result only, because exists/1 is implemented lazily.)'.
	self processQuery: ' if exists(and(equals(?X,7),equals([7 + ?X],?Y)))'
		description:
			'Does exists/1 work when it contains a Smalltalk term containing a logic variable'! !

!LogicProgramControlTest methodsFor: 'meta programming' stamp: 'sp 24/10/11 15:26'!
testexists2
	"self run:#testexists2"

	"exists/2"

	
	self processQuery:
			' if succeeds(exists(equals(?X,7),equals([7 + ?X],?Y)))' description:
			'Does exists/1 work when it contains a Smalltalk term containing a logic variable'.
	self processQuery:
			' if succeeds(exists(member(?X,<1,2,3,4>),greater(?X,3)))' description:
			'Does there exist a member ?X of a list <1,2,3,4> that is strictly greater than 3?'! !

!LogicProgramControlTest methodsFor: 'meta programming' stamp: 'sp 24/10/11 15:26'!
testfails
	"self run:#testfails"

	"fails/1"

	
	self processQuery: ' if succeeds(fails(list([1])))' description:
			'Does fails succeed in case of a negative result?'.
	self processQuery: ' if fails(fails(list(<>)))' description:
			'Does fails fail in case of a positive result?'! !

!LogicProgramControlTest methodsFor: 'sets of solutions' stamp: 'sp 24/10/11 15:26'!
testfindall
	"self run:#testfindall"

	"findall/3"

	
	self processQuery:
			' if succeeds(and(findall(?x,member(?x,<a,b,c>),?L),length(?L,[3])))'
		description: 'Does findall generate a list of all results of some query?'.
	self processQuery: ' if succeeds(findall(?X,member(?X,<a,b,c>),<a,b,c>))'
		description: 'Does findall/3 produced the correct list of results?'.
	self processQuery: ' if fails(findall(?X,member(?X,<a,b,c>),<?>))'
		description:
			'Can findall/3 be used to test whether there is only one result of a query?'.
	self processQuery:
			' if succeeds(findall(?x,and(member(?x,<a,b,c>),integer(?x)),<>))'
		description:
			'Does findall generate an empty list when the tested query fails?'.
	self processQuery:
			' if succeeds(and(equals(?numbers,<[1],[2],[3]>),equals(?names,<a,b,c>),findall(pair(?x,?name),and(member(?x,?numbers),member(?name,?names)),?L),sameElements(?L,<pair([1],a),pair([2],a),pair([3],a),pair([1],b),pair([2],b),pair([3],b),pair([1],c),pair([2],c),pair([3],c)>)))'
		description:
			'Can findall be used to generate a  list of couples of solutions?'! !

!LogicProgramControlTest methodsFor: 'sets of solutions' stamp: 'sp 24/10/11 15:26'!
testforall1
	"self run: #testforall1"

	"forall/1"

	
	self processQuery:
			' if succeeds(forall(and(member(?x,<[1],[2],[3]>),integer(?x))))'
		description:
			'For every member of a list of integers, is it indeed an integer?'.
	self processQuery: ' if fails(forall(member(?x,<>)))' description:
			'When the query to be tested is false, forall/1 should succeed. Indeed, in that case: for all solutions that satisfy this query (that is none) the true condition will succeed.'! !

!LogicProgramControlTest methodsFor: 'sets of solutions' stamp: 'sp 24/10/11 15:26'!
testforall2
	"self run: #testforall2"

	"self debug: #testforall2"

	"forall/2"

	
	self processQuery: ' if succeeds(forall(member(?x,<[1],[2],[3]>),[true]))'
		description:
			'For every member of a list of integers, does the trivial test succeed?'.
	self processQuery: ' if fails(forall(member(?x,<[1],[2],[3]>),[false]))'
		description:
			'When the condition to be tested is false, the query should clearly fail'.
	self processQuery: ' if succeeds(forall(member(?x,<>),[false]))'
		description: 'Unless when forall works on an empty set'.
	self processQuery:
			' if succeeds(forall(member(?x,<[1],[2],[3]>),integer(?x)))'
		description:
			'For every member of a list of integers, is it indeed an integer?'.
	self processQuery:
			' if succeeds(forall(member(?c,<a,a,a>),and(nonvar(?c),equals(?c,a))))'
		description:
			'Is every member of a list of symbols a equal to the symbol a?'.
	self processQuery:
			' if fails(forall(member(?c,<a,b,c>),and(nonvar(?c),equals(?c,a))))'
		description:
			'Is every member of a list of different symbols equal to the symbol a?'.
	self processQuery: ' if succeeds(forall(member(?x,<>),writeLn(?x)))'
		description:
			'Does forall/2 work when the first argument produces no results?'.
	self processQuery:
			' if succeeds(forall(member(?x,<[1],[2],[3]>),writeLn(?x)))'
		description: 'Can forall/2 be used for its side-effects?'.
	self processQuery:
			' if equals(?coll,[OrderedCollection new]),forall(member(?x,<[1],[2]>),[?x < 3])'
		description:
			'Can forall/2 have smalltalkblocks in the testquery with variables from the first query?'.
	self processQuery:
			' if succeeds(and(forall(member(?x,<[1],[2],[3]>),writeLn(?x)),var(?x)))'
		description:
			'After calling a forall/2, are all the explicit internal variables made unbound again?'.
	self processQuery:
			' if succeeds(and(forall(member(?x,<1,2,3>),equals(?x,?y)),var(?x),var(?y)))'
		description:
			'After calling a forall/2, are all the explicit and implicit internal variables made unbound again?'.
	self processQuery:
			' if succeeds(and(forall(fact1(?AString),writeLn(?AString)),var(?AString)))'
		description:
			'Does this print on the transcript a, a, a, b, b, c each separated by a newline and is the variable ?AString unbound after calling forall/2?'.
	self processQuery:
			' if succeeds(and(forall(fact1(?AString),writeLn(?AString)),var(?AString)))'
		description:
			'Does this print on the transcript a, a, a, b, b, c each separated by a newline and is the variable ?AString unbound after calling forall/2?'.
	self processQuery:
			' if succeeds(and(forall(fact1(?AString),and(write(?AString),writeLstLn(<foo,bar>))),var(?AString)))'
		description:
			'Does this print on the transcript afoobar, afoobar, afoobar, bfoobar, bfoobar, cfoobar each separated by a newline and is the variable ?AString unbound after calling forall/2?'.
	self processQuery:
			' if succeeds(and(forall(fact1(?AString),and(findall(?S,and(fact1(?S),equals(?S,?AString)),?ASet),writeLn(?AString))),var(?AString),var(?ASet)))'
		description:
			'Does this print on the transcript a, a, a, b, b, c each separated by a newline and are the variables ?AString and ?ASet unbound after calling forall/2?'.
	self processQuery:
			' if succeeds(and(forall(fact1(?AString),and(findall(?S,and(fact1(?S),equals(?S,?AString)),?ASet),write(?AString),writeLstLn(<foo,bar>))),var(?AString),var(?ASet)))'
		description:
			'Does this print on the transcript afoobar, afoobar, afoobar, bfoobar, bfoobar, cfoobar each separated by a newline and are the variables ?AString and ?ASet unbound after calling forall/2?'.
	self processQuery: ' if succeeds(forall(equals(?x,1),[?x > 0]))'
		description:
			'Does forall work when the test is a smalltalk term containing variables generated by the query?'.
	self processQuery: ' if succeeds(forall(equals(?x,?y),not(ground(?x))))'
		description:
			'Does forall work when the query generates solutions containing unbound variables that are used in the test?'.
	self processQuery:
			' if counterincr(?c),not(forall(member(?x,<a,a,a,b,a,c,c,c>),and(counterincr(?c),equals(?x,a)))),countervalue(?c,5)'
		description:
			'Is the evaluation of forall shortcut so that as soon as the test fails for one of the query''s solutions, the other solutions are not tested?'! !

!LogicProgramControlTest methodsFor: 'sets of solutions' stamp: 'sp 24/10/11 15:26'!
testforall3
	"self run:#testforall3"

	"forall/3"

	
	self processQuery:
			' if succeeds(and(forall(member(?e,<[1],[2],[3]>),equals(?e,[2]),?L),length(?L,2)))'
		description:
			'forall/3 is not lazy and produces a list containing as many elements as there are failures'.
	self processQuery:
			' if succeeds(and(forall(member(?e,<[1],[2],[3]>),equals(?e,[2]),?L),equals(?L,<equals([1],[2]),equals([3],[2])>)))'
		description: 'forall/3 correctly produces all failures'.
	self processQuery:
			' if succeeds(forall(member(?e,<[1],[2],[3]>),integer(?e),<>))'
		description: 'forall/3 produces an empty list when all test succeed'! !

!LogicProgramControlTest methodsFor: 'sets of solutions' stamp: 'sp 24/10/11 15:26'!
testforeach
	"self run: #testforeach"

	"foreach/2"

	
	self processQuery: ' if succeeds(foreach(<1,2,3>,write))' description:
			'Does this print 123 on the Transcript?'! !

!LogicProgramControlTest methodsFor: 'meta programming' stamp: 'sp 24/10/11 15:26'!
testifThenElse
	"self run:#testifThenElse"

	"ifThenElse/3"

	
	self processQuery:
			' if succeeds(and(equals(?X,0),ifThenElse(smaller(?X,1),equals(?Result,trueCase),equals(?Result,falseCase)),equals(?Result,trueCase)))'
		description: 'Does ifThenElse/3 succeed for the true case?'.
	self processQuery:
			' if succeeds(and(equals(?X,2),ifThenElse(smaller(?X,1),equals(?Result,trueCase),equals(?Result,falseCase)),equals(?Result,falseCase)))'
		description: 'Does ifThenElse/3 succeed for the false case?'! !

!LogicProgramControlTest methodsFor: 'sets of solutions' stamp: 'sp 24/10/11 15:26'!
testnodup
	"self run:#testnodup"

	"nodup/1"

	
	self
		processQuery: ' if nodup(member(?x,<1,2,3,1>))'
		resultsCheck: [:results | results size = 3]
		description:
			'Does nodup/1 behave as expected when query contains free variables?'.
	self processQuery: ' if fails(nodup(member(4,<1,2,3,1>)))' description:
			'Does nodup/1 fail when the tested query fails?'! !

!LogicProgramControlTest methodsFor: 'meta programming' stamp: 'sp 24/10/11 15:26'!
testone
	"self run:#testone"

	"one/1"

	
	self processQuery: ' if succeeds(one(list(<>)))' description:
			'Does there exist exactly one empty list?'.
	self processQuery: ' if fails(one(list([1])))' description:
			'Is a number not a list?'.
	self
		processQuery: ' if one(and(member(?x,<1,2,3>),equals(?x,1)))'
		resultsCheck: [:results | results size = 1]
		description:
			'Retrieve exactly one item of a certain value from some list?'.
	self
		processQuery: ' if one(and(member(?x,<2,1,3,1>),equals(?x,1)))'
		resultsCheck: [:results | results size = 1]
		description:
			'Retrieve exactly one item of a certain value from some list, where the list is allowed to contain duplicates of that value?'.
	self processQuery:
			' if succeeds(findall(?X,one(member(1,<2,1,3,1>)),<?One>))' description:
			'Does findall/3 produced a singleton list, when the tested predicate is wrapped in a one/1 statement?'.
	self processQuery: ' if one(and(equals(?X,7),equals([7 + ?X],?Y)))'
		description:
			'Does one work when it contains a Smalltalk term containing a logic variable'! !

!LogicProgramControlTest methodsFor: 'meta programming' stamp: 'sp 24/10/11 15:26'!
testsucceeds
	"self run:#testsucceeds"

	"succeeds/1"

	
	self processQuery: ' if fails(succeeds(list([1])))' description:
			'Does succeeds fail in case of a negative result?'.
	self processQuery: ' if succeeds(succeeds(list(<>)))' description:
			'Does succeeds succeed in case of a positive result?'.
	self processQuery: ' if succeeds(and(equals(?X,7),equals([7 + ?X],?Y)))'
		description:
			'Does succeeds work when it contains a Smalltalk term containing a logic variable'! !

!LogicProgramControlTest methodsFor: 'meta programming' stamp: 'sp 24/10/11 15:26'!
testxor
	"self run:#testxor"

	"xor/2"

	
	self processQuery: ' if succeeds(xor(integer([1]),var([1])))' description:
			'Does xor/2 succeed when first argument succeeds and second fails?'.
	self processQuery: ' if succeeds(xor(integer(?x),var(?x)))' description:
			'Does xor/2 succeed when first argument fails and second succeeds?'.
	self processQuery: ' if fails(xor(integer([1]),positiveInteger([1])))'
		description: 'Does xor/2 fail when both arguments are true?'.
	self processQuery: ' if fails(xor(zero([1]),positiveInteger([-1])))'
		description: 'Does xor/2 fail when both arguments fail?'.
	self processQuery:
			' if fails(xor(equals(?x,[#first]),equals(?y,[#second])))' description:
			'Does xor/2 when both arguments succeed?'.
	self processQuery:
			' if succeeds(and(equals(?x,[#dummy]),xor(equals(?x,[#first]),equals(?y,[#second])),nonvar(?x),equals(?x,[#dummy]),nonvar(?y),equals(?y,[#second])))'
		description:
			'Can xor/2 be used to select the second of two instructions?'! !

LogicTests subclass: #LogicSLGTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOUL-Tests'!

!LogicSLGTests class methodsFor: 'instance creation' stamp: 'sp 24/10/11 15:26'!
new
	"Answer a newly created and initialized instance."

	
	^super new initialize! !

!LogicSLGTests methodsFor: 'testing' stamp: 'AndyKellens 11/2/2011 09:53'!
testBasicTransformations
	
	| tabledRules generated callGraph |
	SoulFactory resetSLGTables.
	tabledRules := OrderedCollection new.
	tabledRules
		addAll:
			(SoulParser parse:
					'tabled tabledpath(?x,?y) if 
			tabledpath(?x,?z),edge(?z,?y).
		tabled tabledpath(?x,?y) if 
			edge(?x,?y)') clauses.
	callGraph := CallGraph withClauses: tabledRules.
	generated := TabledRule transformTabledRules: tabledRules withCallGraph:
			callGraph.
	self
		assert:
			(generated
				contains:
					[:rule | 
					(rule asString reject: [:x | x = Character cr]) matchesRegex:
							'tabledpath\(\?x,\?y\)\s+if\s+\s*\s*slg\(tabledpath\(\?x,\?y\)\)']).
	self
		assert:
			(generated
				contains:
					[:rule | 
					(rule asString reject: [:x | x = Character cr]) matchesRegex:
							'slgtabledpath\(\?Id,\?x,\?y\)\s+if\s+slgcall\(\?Id,<\?x,\?y>,tabledpath\(\?x,\?z\),\s*\[.*\]\s*,\s*cont0tabledpath\)']).
	self
		assert:
			(generated
				contains:
					[:rule | 
					(rule asString reject: [:x | x = Character cr]) matchesRegex:
							'cont0tabledpath\(\?Id,<\?x,\?y>,tabledpath\(\?x,\?z\),\?D\)\s+if\s+edge\(\?z,\?y\),\s*answer\(\?Id,tabledpath\(\?x,\?y\),\?D\)']).
	self
		assert:
			(generated
				contains:
					[:rule | 
					(rule asString reject: [:x | x = Character cr]) matchesRegex:
							'slgtabledpath\(\?Id,\?x,\?y\)\s+if\s+edge\(\?x,\?y\),\s*answer\(\?Id,tabledpath\(\?x,\?y\),\[.*\]\)'])! !

!LogicSLGTests methodsFor: 'testing' stamp: 'AndyKellens 11/3/2011 10:59'!
testCasesFromPaper
	
	| basicPathBlock |
	SoulFactory resetSLGTables.
	basicPathBlock := [:results | 
	(results resultsForVariableNamed: 'X') size = 3
		and:
			[((results resultsForVariableNamed: 'X') collect: [:x | x smalltalkValue])
				asOrderedCollection sort = #(#b #c #d) asOrderedCollection]].
	self processQuery: ' if tabledpath(a,?X)' resultsCheck: basicPathBlock
		description: 'test all the paths from a, first case from paper'.
	self
		processQuery: ' if tabledpath(aa,?X)'
		resultsCheck:
			[:results | (results resultsForVariableNamed: 'X') size = 268]
		description:
			'test all the paths from aa, first case from paper (should take a couple of seconds)'.
	self processQuery: ' if drpath(a,?X)' resultsCheck: basicPathBlock
		description:
			'tests all the paths from a, using the double recursive predicate. Used to test the passing around of variables, 2nd case of the paper'.
	self
		processQuery: ' if diff(?X)'
		resultsCheck:
			[:results | 
			(results resultsForVariableNamed: 'X') size = 2
				and:
					[((results resultsForVariableNamed: 'X') collect: [:x | x smalltalkValue])
						asOrderedCollection sort = #(#b #c) asOrderedCollection]]
		description:
			'test 3rd case of the paper with negation (not through recursion)'.
	self
		processQuery: ' if win(?X)'
		resultsCheck:
			[:results | 
			(results resultsForVariableNamed: 'X') size = 1
				and:
					[((results resultsForVariableNamed: 'X') collect: [:x | x smalltalkValue])
						asOrderedCollection sort = #(#c) asOrderedCollection]]
		description:
			'test 4th case of the paper with negation (through recursion)'! !

!LogicSLGTests methodsFor: 'testing' stamp: 'AndyKellens 11/2/2011 09:53'!
testCasesFromXSB
	
	SoulFactory resetSLGTables.
	self
		processQuery: ' if mutRetA(?X,?Y)'
		resultsCheck: [:results | results size = 8]
		description: 'mutual recursion'! !

!LogicSLGTests methodsFor: 'testing' stamp: 'AndyKellens 11/2/2011 09:53'!
testNotTransformations
	
	| tabledRules generated callGraph |
	SoulFactory resetSLGTables.
	tabledRules := OrderedCollection new.
	tabledRules
		addAll:
			(SoulParser parse:
					'tabled tabledpath(?x,?y) if 
			tabledpath(?x,?z),edge(?z,?y).
		tabled tabledpath(?x,?y) if 
			edge(?x,?y).
		tabled diff(?N) if 
			tabledpath(a,?N),not(tabledpath(c,?N))') clauses.
	callGraph := CallGraph withClauses: tabledRules.
	generated := (tabledRules at: 3) slgTransformWith: tabledRules
		withCallGraph: callGraph.
	self
		assert:
			(generated
				contains:
					[:rule | 
					(rule asString reject: [:x | x = Character cr]) matchesRegex:
							'slgdiff\(\?Id,\?N\)\s+if\s+slgcall\(\?Id,<\?N>,tabledpath\(a,\?N\),\[.*\],cont0diff\)']).
	self
		assert:
			(generated
				contains:
					[:rule | 
					(rule asString reject: [:x | x = Character cr]) matchesRegex:
							'cont0diff\(\?Id,<\?N>,tabledpath\(a,\?N\),\?D\)\s+if\s+slgnot\(\?Id,<\?N>,tabledpath\(c,\?N\),\[.*\],cont1diff\)']).
	self
		assert:
			(generated
				contains:
					[:rule | 
					(rule asString reject: [:x | x = Character cr]) matchesRegex:
							'cont1diff\(\?Id,<\?N>,\?D\)\s+if\s+answer\(\?Id,diff\(\?N\),\?D\)'])! !

!LogicSLGTests methodsFor: 'testing' stamp: 'AndyKellens 11/2/2011 09:53'!
testSLGEquivalence
	
	SoulFactory resetSLGTables.
	self
		processQuery: ' if shortpath(?X,?X)'
		resultsCheck:
			[:results | (results resultsForVariableNamed: 'X') size = 4]
		description: 'test all cyclic paths'.
	self
		processQuery: ' if shortpath(?X,?Y)'
		resultsCheck:
			[:results | (results resultsForVariableNamed: 'X') size = 16]
		description:
			'test that this call does not returns the results from ?X,?X'! !

!LogicTests class methodsFor: 'Examples' stamp: 'sp 24/10/11 15:26'!
averageTotalOfRuns: number
	"Run me a certain times, and return the average time it took"

	"QSOULArithmeticTest averageTotalOfRuns: 3"

	
	| allTotals theTimer suite |
	allTotals := OrderedCollection new.
	theTimer := TotalTimer new.
	self setLogicTimer: theTimer.
	suite := self buildSuite.
	number
		timesRepeat:
			[suite run.
			allTotals add: theTimer totalResult.
			theTimer resetTimings].
	^(allTotals
		inject: 0
		into: [:subTotal :timing | subTotal + timing]) / number! !

!LogicTests class methodsFor: 'Logic Timer' stamp: 'sp 24/10/11 15:26'!
defaultLogicTimer
	"returns the default notifier, that does no timing."

	
	^LogicTestNotifier new! !

!LogicTests class methodsFor: 'Examples' stamp: 'sp 24/10/11 15:26'!
fullyTimeSuite
	"Fully time me, and output the results on the default stream."

	
	| theTimer |
	theTimer := TotalTimer new.
	self setLogicTimer: theTimer.
	self buildSuite run.
	theTimer outputClassResults.
	theTimer outputTotalResult! !

!LogicTests class methodsFor: 'Examples' stamp: 'sp 24/10/11 15:26'!
fullyTimeSuiteOnFileNamed: aFileNameString
	"Fully time me, and output the results on a file with the given name."

	
	| stream |
	stream := aFileNameString asFilename writeStream.
	[self fullyTimeSuiteOnStream: stream] ensure: [stream close]! !

!LogicTests class methodsFor: 'Examples' stamp: 'sp 24/10/11 15:26'!
fullyTimeSuiteOnStream: aStream
	"Fully time me, and output the results on the given stream."

	
	| theTimer |
	theTimer := TotalTimer new.
	theTimer outputStream: aStream.
	self setLogicTimer: theTimer.
	self buildSuite run.
	theTimer outputClassResults.
	theTimer outputTotalResult! !

!LogicTests class methodsFor: 'Examples' stamp: 'sp 24/10/11 15:26'!
fullyTimeSuitePromptForFile
	"for one class: QSOULArithmeticTest fullyTimeSuitePromptForFile"

	"For all: QSOULLogicTests fullyTimeSuitePromptForFile"

	"Prompt for a file name, fully time me, and output the results on a file with the given name."

	
	| fileName |
	fileName := Dialog
		requestFileName: 'Give the name of the file to write the results on:'
		default: 'testTimings.tsf'
		version: #new
		ifFail:
			[^Dialog warn:
					'Not testing anything since there is no file to save the results.'].
	self fullyTimeSuiteOnFileNamed: fileName! !

!LogicTests class methodsFor: 'Logic Timer' stamp: 'sp 24/10/11 15:26'!
initializeLogicTimer
	"initializes the logic timer."

	
	self setLogicTimer: self defaultLogicTimer! !

!LogicTests class methodsFor: 'Testing' stamp: 'sp 24/10/11 15:26'!
isAbstract
	"Override to true if a TestCase subclass is Abstract and should not have
	TestCase instances built from it"

	"Overridden because that is the case here."

	
	^self name = #LogicTests! !

!LogicTests class methodsFor: 'configuration' stamp: 'sp 24/10/11 15:26'!
isDetailedTimed
	"this setting determines whether individual tests are timed, and the  
	results displayed on the Transcript. By default this is false (so  
	it runs as plain tests, without timing). Set this to true to allow fine  
	grained benchmarking."

	
	IsDetailedTimed isNil ifTrue: [IsDetailedTimed := true].
	^IsDetailedTimed! !

!LogicTests class methodsFor: 'configuration' stamp: 'sp 24/10/11 15:26'!
isSelectorTimed
	"this setting determines whether the user specific timings are executed 
	and  
	shown (see the methods #startWatch and #stopAndShowWatch). If true,  
	these results are displayed on the Transcript. By default this is false (so 
	it runs as plain tests, without timing). Set this true to allow course  
	grained benchmarking."

	
	IsSelectorTimed isNil ifTrue: [IsSelectorTimed := false].
	^IsSelectorTimed! !

!LogicTests class methodsFor: 'Logic Timer' stamp: 'sp 24/10/11 15:26'!
logicTimer
	"Return the current tmer being used."

	
	LogicTimer isNil ifTrue: [self initializeLogicTimer].
	^LogicTimer! !

!LogicTests class methodsFor: 'configuration' stamp: 'sp 24/10/11 15:26'!
setDetailedTimed
	"self setDetailedTimed"

	"this setting determines that every individual tests is timed, and that  
	these results are displayed on the Transcript"

	
	IsDetailedTimed := true! !

!LogicTests class methodsFor: 'Logic Timer' stamp: 'sp 24/10/11 15:26'!
setLogicTimer: aLogicTimer
	"Sets the logic timer to the argument."

	
	LogicTimer := aLogicTimer! !

!LogicTests class methodsFor: 'configuration' stamp: 'sp 24/10/11 15:26'!
setNotDetailedTimed
	"self setNotDetailedTimed"

	"this setting determines that detailed timings are not displayed"

	
	IsDetailedTimed := false! !

!LogicTests class methodsFor: 'configuration' stamp: 'sp 24/10/11 15:26'!
setNotSelectorTimed
	"self setNotSelectorTimed"

	"this setting determines that no timings per selector are shown"

	
	IsSelectorTimed := false! !

!LogicTests class methodsFor: 'configuration' stamp: 'sp 24/10/11 15:26'!
setSelectorTimed
	"self setSelectorTimed"

	"show timings per test selector executed"

	
	IsSelectorTimed := true! !

!LogicTests methodsFor: 'Private' stamp: 'sp 24/10/11 15:26'!
currentTotal
	
	currentTotal isNil ifTrue: [currentTotal := 0].
	^currentTotal! !

!LogicTests methodsFor: 'Accessing' stamp: 'sp 24/10/11 15:26'!
logicTimer
	
	^self class logicTimer! !

!LogicTests methodsFor: 'Assertions' stamp: 'sp 24/10/11 15:26'!
processQuery: aString description: descriptionString
	"This method is exactly the same as processQuery:resultsCheck:description: 
	except that the resultcheck is left out. It is replaced by a default check 
	that the result was successful."

	
	self
		processQuery: aString
		resultsCheck: [:results | results success]
		description: descriptionString! !

!LogicTests methodsFor: 'Assertions' stamp: 'sp 24/10/11 15:26'!
processQuery: aString description: descriptionString repository: aRepository
	"This method is exactly the same as processQuery:resultsCheck:description: 
	except that the resultcheck is left out. It is replaced by a default check 
	that the result was successful."

	
	self
		processQuery: aString
		resultsCheck: [:results | results success]
		description: descriptionString
		repository: aRepository! !

!LogicTests methodsFor: 'Assertions' stamp: 'sp 24/10/11 15:26'!
processQuery: aString resultsCheck: checkBlock description: descriptionString
	
	self
		processQuery: aString
		resultsCheck: checkBlock
		description: descriptionString
		repository: self repository! !

!LogicTests methodsFor: 'Assertions' stamp: 'AndyKellens 11/2/2011 09:53'!
processQuery: anId resultsCheck: checkBlock description: descriptionString repository: aRepository
	"Note that the description is not used here, but is meant for subclasses  
	that allow to log extra information (for example to show the time it took 
	to process a query)."

	
	| results |
	"aString := self queryWithId: anId.
	self logicTimer beforeQuery: aString."
	results := (SoulFactory current evaluatorForLogicQuery eval: anId in:
			aRepository) allResults.
	self assert: (checkBlock value: results)	"self logicTimer afterQuery: results description: descriptionString"! !

!LogicTests methodsFor: 'Accessing' stamp: 'AndyKellens 11/2/2011 09:53'!
repository
	
	^SoulFactory testRepository! !

!LogicTests methodsFor: 'Private' stamp: 'sp 24/10/11 15:26'!
resetTotal
	"set the current total to 0"

	
	currentTotal := 0! !

!LogicTests methodsFor: 'Running' stamp: 'sp 24/10/11 15:26'!
runCase
	"Run the case. Overridden to inform the notifier before and after this happens."

	
	self logicTimer beforeCaseDo: self.
	super runCase.
	self logicTimer afterCaseDo: self! !

!LogicTests methodsFor: 'Private' stamp: 'sp 24/10/11 15:26'!
runCaseSelectorTimed
	"like the general runcase, but timed"

	
	self setUp.
	[self resetTotal.
	self perform: testSelector sunitAsSymbol.
	Transcript
		show: testSelector;
		show: ': took ';
		print: self currentTotal;
		show: ' ms';
		cr] sunitEnsure: [self tearDown]! !

LogicTests subclass: #SoulReflectionTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOUL-Tests'!

!SoulReflectionTest methodsFor: 'repositories' stamp: 'sp 24/10/11 15:26'!
testexistsInRepository
	"self run:#testexistsInRepository"

	"existsInRepository/1"

	
	self processQuery:
			' if succeeds(existsInRepository(fact2(onlyOccurence)))' description:
			'Can we find a clause in the repository of which the head unifies with fact2(onlyOccurence)'.
	self processQuery: ' if fails(existsInRepository(fact2(zork)))'
		description:
			'Cannot we find a clause in the repository of which the head unifies with fact2(zork)'.
	self processQuery: ' if succeeds(existsInRepository(fact2(?)))'
		description:
			'Can we find a clause in the repository of which the head unifies with fact2(?)'.
	self
		processQuery: ' if existsInRepository(fact1(?x))'
		resultsCheck: [:results | results size = 1]
		description:
			'If there is more than one matching clause in the repository, do we get only one result?'! !

LogicTests subclass: #SymbiosisTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOUL-Tests'!

!SymbiosisTest methodsFor: 'generate' stamp: 'sp 24/10/11 15:26'!
testcollectionMember
	"self run:#testcollectionMember"

	
	self
		processQuery: ' if collectionMember(?x,[#(1 2 3) asOrderedCollection])'
		resultsCheck: [:results | results size = 3]
		description: 'Check the pure generative function'.
	self
		processQuery:
			' if equals(?x,dummy),collectionMember(1,[#(1 2 3) asOrderedCollection])'
		resultsCheck: [:results | results size = 1]
		description: 'Can we also use a non-variable as first argument?'.
	self
		processQuery: ' if collectionMember(?x,[#(1 2)])'
		resultsCheck: [:results | results size = 2]
		description: 'Tests whether collectionMember can be used on an Array'.
	self
		processQuery:
			' if collectionMember(?x,[#(#(1) #(2))]),collectionMember(?y,?x)'
		resultsCheck: [:results | results size = 2]
		description:
			'Tests whether collectionMember can be used on the result of a collectionMember call.'.
	self
		processQuery:
			' if collectionMember(?x,[#(#(1) #(2))]),collectionMember(?y,[?x])'
		resultsCheck: [:results | results size = 2]
		description:
			'Tests whether collectionMember can be used on the result of a collectionMember call when a Smalltalk term is used to pass the second collection. (note: this test has been used to expose a particularly nasty problem in Squeak caused by Squeak''s limitations on blocks. KG)'.
	self
		processQuery:
			' if collectionMember(?x,[#(#(1 2 3) #(4 5 6) #(7 8 9 10))]),collectionMember(?y,[?x])'
		resultsCheck: [:results | results size = 10]
		description: 'Tests a more extensive variation of the above test.'! !

!SymbiosisTest methodsFor: 'generate' stamp: 'sp 24/10/11 15:26'!
testmember
	"self run:#testmember"

	
	self
		processQuery: ' if member(?x,[#(1 2 3) asOrderedCollection])'
		resultsCheck: [:results | results size = 3]
		description: 'Check the pure generative function'.
	self
		processQuery:
			' if equals(?x,dummy),member(1,[#(1 2 3) asOrderedCollection])'
		resultsCheck: [:results | results size = 1]
		description: 'Can we also use a non-variable as first argument?'! !

!SymbiosisTest methodsFor: 'quotedCode' stamp: 'sp 24/10/11 15:26'!
testsourceString
	"self run:#testsourceString"

	
	self processQuery: ' if sourceString({simpele test},[''simpele test''])'
		description: 'Is a quotedcode converted correctly?'.
	self processQuery:
			' if sourceString({simpele test},?c),equals(?c,[''simpele test''])'
		description: 'Is a quotedcode converted correctly?'.
	self processQuery:
			' if sourceString(?c,[''simpele test'']),equals(?c,{simpele test})'
		description: 'Is a string converted correctly?'! !

LogicTests subclass: #TransparentSymbiosisTest
	instanceVariableNames: 'oldFactory'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOUL-Tests'!

!TransparentSymbiosisTest methodsFor: 'Running' stamp: 'AndyKellens 11/2/2011 09:53'!
setUp
	
	oldFactory := SoulFactory current.
	SymbiosisFactory installSymbiosisFactory! !

!TransparentSymbiosisTest methodsFor: 'Running' stamp: 'AndyKellens 10/28/2011 10:56'!
tearDown
	
	SymbiosisFactory revertTo: oldFactory! !

!TransparentSymbiosisTest methodsFor: 'smalltalk -> soul' stamp: 'sp 24/10/11 15:26'!
testCorrectFailure
	"Tests whether it doesn't get into an endless loop when neither rule nor method exists. Placed seperately just in case it *does* loop so you know what's wrong."

	
	self
		should:
			[TransparentSymbiosisTestClass new supercalifragilisticexpialidocious]
		raise: Exception! !

!TransparentSymbiosisTest methodsFor: 'smalltalk -> soul' stamp: 'sp 24/10/11 15:26'!
testSmalltalk2Soul
	
	| symbiosisObject |
	symbiosisObject := TransparentSymbiosisTestClass new.	"Test whether the predicate testValueFor= is invoked and whether the result is correctly returned as an orderedcollection"

	self
		assert:
			(symbiosisObject testValueFor: 1)
				= (OrderedCollection with: 1 with: 11 with: 111).	"Test whether a single value is returned as a FakeSingleItemCollection, meaning it forwards messages to the item"

	self assert: (symbiosisObject testValueFor: 2) + 1 = 3.	"Test whether a single value is returned as a FakeSingleItemCollection, meaning it can respond to some basic Collection protocol messages"

	self assert: (symbiosisObject testValueFor: 2) first + 1 = 3.	"Test whether predicates without a = can be correctly invoked (note: the explicit = true is intentional, in case the message before it returns a non-boolean)"

	self assert: (symbiosisObject supportsTestValueFor: 2) = true.	"Variation on the previous one."

	self assert: (symbiosisObject supportsTestValueFor: 3) = false.	"Tests the use of uninitialized temporaries as arguments (in the TransparentSymbiosisTestClass method that is called here)"

	self
		assert:
			symbiosisObject valuesWithTestValues
				= (OrderedCollection with: 1 with: 1 with: 2 with: 1).	"Tests the sending of a message to an unitialized temporary"

	self
		assert:
			symbiosisObject valuesThatTestFor = (OrderedCollection with: 2 with: 1).	"Tests wether a query that results in just multiple true value returns just true instead of a collection of true"

	self assert: symbiosisObject trueRule = true! !

!TransparentSymbiosisTest methodsFor: 'soul -> smalltalk' stamp: 'AndyKellens 10/28/2011 13:35'!
testSoul2Smalltalk
	"These tests aren't very good because they still depend on the old symbiosis mechanism with the brackets .... it should perhaps become possible in the future to specify class names in SOUL directly without having to use the brackets"

	
	self processQuery: ' if [Array new] isEmpty' description:
			'Tests if a MessageTerm is correctly translated to a message'.
	self processQuery:
			' if [OrderedCollection new] asString = [''an OrderedCollection()'']'
		description:
			'Tests if a ResultMessageTerm without arguments is correctly translated to a message'.
	self processQuery: ' if [Array] with: 1 with: 2 = ?o,?o includes: 1'
		description:
			'Tests if a ResultMessageTerm with arguments is correctly translated to a message'.
	self processQuery:
			' if equals(?x,[OrderedCollection with: 1 with: 2 with: 3]),findall(?r,?x yourself = ?r,<1,2,3>)'
		description:
			'Tests wether results of messages to Smalltalk that are contained in an OrderedCollection are unwrapped and returned as seperate results.'.
	self processQuery:
			' if equals(?x,[OrderedCollection with: 1 with: 2 with: 3]),?x yourself = ?r,equals(?r,3),findall(?s,?x yourself = ?s,<1,2,3>)'
		description:
			'Tests wether the collections returned from messages to Smalltalk are not modified due to backtracking.'! !

SymbiosisClass subclass: #TransparentSymbiosisTestClass
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOUL-Tests'!

!TransparentSymbiosisTestClass methodsFor: 'test' stamp: 'AndyKellens 10/28/2011 14:54'!
valuesThatTestFor
	| values x |
	x := UninitializedTemporary new.
	values := UninitializedTemporary new.
	values testsFor: x.
	^ values! !

!TransparentSymbiosisTestClass methodsFor: 'test' stamp: 'AndyKellens 10/28/2011 14:53'!
valuesWithTestValues
	| values |
	values := UninitializedTemporary new.
	self testValueFor: values.
	^ values! !

RBProgramNodeVisitor subclass: #LogicTestsRenamingVisitor
	instanceVariableNames: 'compoundTermRenamingVisitor didRenaming'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOUL-Tests'!

!LogicTestsRenamingVisitor class methodsFor: 'rewriting' stamp: 'sp 24/10/11 15:26'!
rewriteAllTestsWith: aCompoundTermRenamingVisitor
	
	^(self new)
		compoundTermRenamingVisitor: aCompoundTermRenamingVisitor;
		rewriteAllTests! !

!LogicTestsRenamingVisitor methodsFor: 'visiting' stamp: 'AndyKellens 11/2/2011 09:53'!
acceptMessageNode: msg
	
	| queryString query newQuery |
	super acceptMessageNode: msg.
	('processQuery:*' match: msg selector)
		ifTrue:
			[queryString := msg arguments first value.
			query := SoulFactory current makeParser parse: queryString.
			compoundTermRenamingVisitor resetDidRenaming.
			newQuery := query accept: compoundTermRenamingVisitor.
			compoundTermRenamingVisitor didRenaming
				ifTrue:
					[msg arguments first replaceWith: (RBLiteralNode value: newQuery asString).
					didRenaming := true]]! !

!LogicTestsRenamingVisitor methodsFor: 'initialization' stamp: 'sp 24/10/11 15:26'!
compoundTermRenamingVisitor: aVisitor
	
	compoundTermRenamingVisitor := aVisitor! !

!LogicTestsRenamingVisitor methodsFor: 'rewriting' stamp: 'sp 24/10/11 15:26'!
rewrite: selector in: class
	
	| method |
	didRenaming := false.
	method := RBParser parseMethod: (class sourceCodeAt: selector).
	method acceptVisitor: self.
	didRenaming
		ifTrue:
			[Transcript
				show:
						'Recompiling test case: ' , class asString , '>>' , selector asString;
				cr.
			class compile: method formattedCode]! !

!LogicTestsRenamingVisitor methodsFor: 'rewriting' stamp: 'sp 24/10/11 15:26'!
rewriteAllTests
	
	LogicTests allSubclasses
		do:
			[:testclass | 
			testclass selectors
				do:
					[:selector | 
					Transcript
						show: testclass name , '>>' , selector asString;
						cr.
					self rewrite: selector in: testclass]]! !

LogicStorage subclass: #TestQueriesLayer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOUL-Tests'!

TestQueriesLayer subclass: #TestClauses
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOUL-Tests'!

!TestClauses methodsFor: 'soul auto saved' stamp: 'sp 24/10/11 15:26'!
fooTestM1

fooTest(1).
fooTest(2)! !

!TestClauses methodsFor: 'soul auto saved' stamp: 'sp 24/10/11 15:26'!
testNegVariableM1

testNegVariable(-?x)! !

!TestClauses methodsFor: 'soul auto saved' stamp: 'sp 24/10/11 15:26'!
testNegVariableM2

testNegVariable(?x,-?y) if 
	equals(?x,?y)! !

TestClauses subclass: #TestClauses1
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOUL-Tests'!

!TestClauses1 methodsFor: 'soul auto saved' stamp: 'sp 24/10/11 15:26'!
fact1M1

"Do not change these facts!!!! The result of some unit tests depends on the actual facts defined here"

fact1(a).
fact1(a).
fact1(a).
fact1(b).
fact1(b).
fact1(c)! !

!TestClauses1 methodsFor: 'soul auto saved' stamp: 'sp 24/10/11 15:26'!
fact2M1

"Do not change these facts!!!! The result of some unit tests depends on the actual facts defined here"

fact2(onlyOccurence)! !

!TestClauses1 methodsFor: 'soul auto saved' stamp: 'sp 24/10/11 15:26'!
fact3M1

"Do not change these facts!!!! The result of some unit tests depends on the actual facts defined here.

What we will be testing here is what happened when a set of facts with the same head  is defined in
	 multiple classes: can we see them all, or only those of one of the classes?"

fact3(firstOccurence).
fact3(secondOccurence)! !

!TestClauses1 methodsFor: 'soul auto saved' stamp: 'sp 24/10/11 15:26'!
findalltestM2

findalltest(?list,?results) if 
	findall(result(?foo,?bar),and(member(?foo,?list),member(?bar,?list)),?results)! !

!TestClauses1 methodsFor: 'soul auto saved' stamp: 'sp 24/10/11 15:26'!
fooXbaarXM3

?some foo: ?x baar: ?y! !

!TestClauses1 methodsFor: 'soul auto saved' stamp: 'sp 24/10/11 15:26'!
fooXbarXselXM3

foo: bla bar: ?tokens1 sel: ?tokens3 if 
	{test},foo: bla bar: ?tokens2 sel: ?tokens3.
foo: bla bar: ?tokens1 sel: ?tokens2 if 
	foo(?tokens1,?tokens2)! !

!TestClauses1 methodsFor: 'soul auto saved' stamp: 'sp 24/10/11 15:26'!
fooXbarXtestXM3

foo: bla bar: ?tokens1 test: ?tokens3 if 
	{test},foo: bla bar: ?tokens2 test: ?tokens3.
foo: bla bar: ?tokens1 test: ?tokens2 if 
	foo(?tokens1,?tokens2)! !

!TestClauses1 methodsFor: 'soul auto saved' stamp: 'sp 24/10/11 15:26'!
metaFactM1

metaFact(fact3(firstOccurence))! !

TestClauses subclass: #TestClauses2
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOUL-Tests'!

!TestClauses2 methodsFor: 'soul auto saved' stamp: 'sp 24/10/11 15:26'!
fact3M1

"Do not change these facts!!!! The result of some unit tests depends on the actual facts defined here.

What we will be testing here is what happened when a set of facts with the same head  is defined in
	 multiple classes: can we see them all, or only those of one of the classes?"

fact3(firstOccurenceInOtherClass).
fact3(secondOccurenceInOtherClass)! !

!TestClauses2 methodsFor: 'soul auto saved' stamp: 'sp 24/10/11 15:26'!
testcutonfinalM1

testcutonfinal(a).
testcutonfinal(b).
testcutonfinal(c) if 
	!!! !

TestClauses subclass: #TestClauses3
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOUL-Tests'!

!TestClauses3 methodsFor: 'Dynamic tests' stamp: 'sp 24/10/11 15:26'!
dynamicM1

dynamic(testDynamicPredicate)! !

!TestClauses3 methodsFor: 'soul auto saved' stamp: 'sp 24/10/11 15:26'!
supportsTestValueForXM2

?self supportsTestValueFor: ?y if 
	?self testValueFor: ?y = ?someVal! !

!TestClauses3 methodsFor: 'soul auto saved' stamp: 'sp 24/10/11 15:26'!
testValueForXEM3

?self testValueFor: 1 = 1.
?self testValueFor: 1 = 11.
?self testValueFor: 2 = 2.
?self testValueFor: 1 = 111! !

!TestClauses3 methodsFor: 'soul auto saved' stamp: 'sp 24/10/11 15:26'!
testsForXM2

2 testsFor: 3.
1 testsFor: 2! !

!TestClauses3 methodsFor: 'soul auto saved' stamp: 'sp 24/10/11 15:26'!
trueRuleM1

?o trueRule.
?o trueRule! !

!TestQueriesLayer methodsFor: 'slg performance' stamp: 'sp 24/10/11 15:26'!
cylM2

cyl(1,30).
cyl(1,40).
cyl(2,43).
cyl(2,34).
cyl(3,30).
cyl(3,33).
cyl(4,45).
cyl(4,40).
cyl(5,31).
cyl(5,45).
cyl(6,31).
cyl(6,48).
cyl(7,31).
cyl(7,41).
cyl(8,25).
cyl(8,30).
cyl(9,40).
cyl(9,31).
cyl(10,35).
cyl(10,46).
cyl(11,32).
cyl(11,28).
cyl(12,35).
cyl(12,43).
cyl(13,46).
cyl(13,48).
cyl(14,39).
cyl(14,35).
cyl(15,46).
cyl(15,28).
cyl(16,28).
cyl(16,42).
cyl(17,33).
cyl(17,25).
cyl(18,46).
cyl(18,27).
cyl(19,38).
cyl(19,47).
cyl(20,27).
cyl(20,41).
cyl(21,34).
cyl(21,38).
cyl(22,27).
cyl(22,33).
cyl(23,26).
cyl(23,35).
cyl(24,36).
cyl(24,25).
cyl(25,70).
cyl(25,52).
cyl(26,59).
cyl(26,71).
cyl(27,61).
cyl(27,58).
cyl(28,61).
cyl(28,54).
cyl(29,63).
cyl(29,70).
cyl(30,58).
cyl(30,53).
cyl(31,56).
cyl(31,60).
cyl(32,58).
cyl(32,50).
cyl(33,62).
cyl(33,66).
cyl(34,55).
cyl(34,72).
cyl(35,63).
cyl(35,58).
cyl(36,55).
cyl(36,64).
cyl(37,56).
cyl(37,58).
cyl(38,68).
cyl(38,61).
cyl(39,64).
cyl(39,52).
cyl(40,57).
cyl(40,70).
cyl(41,69).
cyl(41,55).
cyl(42,62).
cyl(42,53).
cyl(43,68).
cyl(43,65).
cyl(44,56).
cyl(44,62).
cyl(45,67).
cyl(45,71).
cyl(46,71).
cyl(46,66).
cyl(47,61).
cyl(47,60).
cyl(48,60).
cyl(48,54).
cyl(49,93).
cyl(49,88).
cyl(50,90).
cyl(50,93).
cyl(51,95).
cyl(51,92).
cyl(52,93).
cyl(52,94).
cyl(53,83).
cyl(53,90).
cyl(54,78).
cyl(54,79).
cyl(55,79).
cyl(55,92).
cyl(56,96).
cyl(56,94).
cyl(57,94).
cyl(57,80).
cyl(58,79).
cyl(58,83).
cyl(59,75).
cyl(59,96).
cyl(60,86).
cyl(60,79).
cyl(61,85).
cyl(61,75).
cyl(62,82).
cyl(62,95).
cyl(63,85).
cyl(63,78).
cyl(64,92).
cyl(64,86).
cyl(65,76).
cyl(65,78).
cyl(66,78).
cyl(66,81).
cyl(67,96).
cyl(67,78).
cyl(68,88).
cyl(68,77).
cyl(69,86).
cyl(69,90).
cyl(70,93).
cyl(70,80).
cyl(71,92).
cyl(71,74).
cyl(72,88).
cyl(72,81).
cyl(73,113).
cyl(73,116).
cyl(74,101).
cyl(74,100).
cyl(75,113).
cyl(75,109).
cyl(76,112).
cyl(76,98).
cyl(77,109).
cyl(77,108).
cyl(78,112).
cyl(78,117).
cyl(79,101).
cyl(79,110).
cyl(80,110).
cyl(80,119).
cyl(81,108).
cyl(81,98).
cyl(82,111).
cyl(82,113).
cyl(83,116).
cyl(83,111).
cyl(84,114).
cyl(84,103).
cyl(85,97).
cyl(85,114).
cyl(86,107).
cyl(86,120).
cyl(87,116).
cyl(87,105).
cyl(88,99).
cyl(88,105).
cyl(89,118).
cyl(89,110).
cyl(90,104).
cyl(90,108).
cyl(91,98).
cyl(91,106).
cyl(92,100).
cyl(92,108).
cyl(93,117).
cyl(93,114).
cyl(94,115).
cyl(94,118).
cyl(95,99).
cyl(95,108).
cyl(96,111).
cyl(96,98).
cyl(97,125).
cyl(97,132).
cyl(98,134).
cyl(98,131).
cyl(99,124).
cyl(99,136).
cyl(100,122).
cyl(100,129).
cyl(101,140).
cyl(101,125).
cyl(102,142).
cyl(102,137).
cyl(103,137).
cyl(103,141).
cyl(104,135).
cyl(104,132).
cyl(105,126).
cyl(105,137).
cyl(106,142).
cyl(106,128).
cyl(107,123).
cyl(107,143).
cyl(108,126).
cyl(108,132).
cyl(109,128).
cyl(109,130).
cyl(110,124).
cyl(110,136).
cyl(111,123).
cyl(111,141).
cyl(112,128).
cyl(112,142).
cyl(113,130).
cyl(113,128).
cyl(114,144).
cyl(114,139).
cyl(115,141).
cyl(115,139).
cyl(116,134).
cyl(116,126).
cyl(117,135).
cyl(117,131).
cyl(118,137).
cyl(118,142).
cyl(119,133).
cyl(119,125).
cyl(120,135).
cyl(120,139).
cyl(121,154).
cyl(121,151).
cyl(122,150).
cyl(122,156).
cyl(123,158).
cyl(123,168).
cyl(124,160).
cyl(124,168).
cyl(125,159).
cyl(125,161).
cyl(126,167).
cyl(126,156).
cyl(127,151).
cyl(127,167).
cyl(128,164).
cyl(128,152).
cyl(129,154).
cyl(129,158).
cyl(130,164).
cyl(130,150).
cyl(131,165).
cyl(131,155).
cyl(132,154).
cyl(132,157).
cyl(133,163).
cyl(133,161).
cyl(134,147).
cyl(134,160).
cyl(135,156).
cyl(135,148).
cyl(136,153).
cyl(136,157).
cyl(137,159).
cyl(137,152).
cyl(138,149).
cyl(138,152).
cyl(139,161).
cyl(139,157).
cyl(140,167).
cyl(140,161).
cyl(141,168).
cyl(141,145).
cyl(142,161).
cyl(142,160).
cyl(143,146).
cyl(143,150).
cyl(144,160).
cyl(144,163).
cyl(145,184).
cyl(145,171).
cyl(146,187).
cyl(146,171).
cyl(147,179).
cyl(147,182).
cyl(148,185).
cyl(148,180).
cyl(149,187).
cyl(149,174).
cyl(150,175).
cyl(150,190).
cyl(151,176).
cyl(151,185).
cyl(152,169).
cyl(152,182).
cyl(153,181).
cyl(153,188).
cyl(154,190).
cyl(154,179).
cyl(155,184).
cyl(155,187).
cyl(156,169).
cyl(156,184).
cyl(157,183).
cyl(157,186).
cyl(158,174).
cyl(158,179).
cyl(159,175).
cyl(159,172).
cyl(160,190).
cyl(160,189).
cyl(161,180).
cyl(161,175).
cyl(162,192).
cyl(162,182).
cyl(163,179).
cyl(163,175).
cyl(164,174).
cyl(164,181).
cyl(165,178).
cyl(165,185).
cyl(166,170).
cyl(166,169).
cyl(167,183).
cyl(167,178).
cyl(168,180).
cyl(168,181).
cyl(169,213).
cyl(169,207).
cyl(170,206).
cyl(170,203).
cyl(171,195).
cyl(171,209).
cyl(172,214).
cyl(172,197).
cyl(173,205).
cyl(173,206).
cyl(174,212).
cyl(174,214).
cyl(175,201).
cyl(175,204).
cyl(176,206).
cyl(176,200).
cyl(177,202).
cyl(177,207).
cyl(178,202).
cyl(178,203).
cyl(179,216).
cyl(179,196).
cyl(180,211).
cyl(180,197).
cyl(181,193).
cyl(181,207).
cyl(182,196).
cyl(182,194).
cyl(183,215).
cyl(183,199).
cyl(184,203).
cyl(184,204).
cyl(185,196).
cyl(185,208).
cyl(186,195).
cyl(186,212).
cyl(187,193).
cyl(187,194).
cyl(188,204).
cyl(188,200).
cyl(189,205).
cyl(189,201).
cyl(190,210).
cyl(190,194).
cyl(191,193).
cyl(191,209).
cyl(192,208).
cyl(192,209).
cyl(193,227).
cyl(193,223).
cyl(194,240).
cyl(194,227).
cyl(195,239).
cyl(195,230).
cyl(196,228).
cyl(196,230).
cyl(197,234).
cyl(197,221).
cyl(198,240).
cyl(198,222).
cyl(199,221).
cyl(199,235).
cyl(200,230).
cyl(200,235).
cyl(201,230).
cyl(201,225).
cyl(202,238).
cyl(202,217).
cyl(203,224).
cyl(203,217).
cyl(204,221).
cyl(204,234).
cyl(205,228).
cyl(205,217).
cyl(206,221).
cyl(206,230).
cyl(207,220).
cyl(207,240).
cyl(208,224).
cyl(208,219).
cyl(209,217).
cyl(209,237).
cyl(210,232).
cyl(210,239).
cyl(211,235).
cyl(211,223).
cyl(212,228).
cyl(212,220).
cyl(213,229).
cyl(213,234).
cyl(214,230).
cyl(214,228).
cyl(215,223).
cyl(215,219).
cyl(216,221).
cyl(216,240).
cyl(217,243).
cyl(217,256).
cyl(218,246).
cyl(218,252).
cyl(219,250).
cyl(219,247).
cyl(220,257).
cyl(220,243).
cyl(221,245).
cyl(221,261).
cyl(222,254).
cyl(222,245).
cyl(223,258).
cyl(223,252).
cyl(224,244).
cyl(224,242).
cyl(225,253).
cyl(225,250).
cyl(226,263).
cyl(226,248).
cyl(227,251).
cyl(227,262).
cyl(228,249).
cyl(228,248).
cyl(229,258).
cyl(229,257).
cyl(230,258).
cyl(230,256).
cyl(231,262).
cyl(231,254).
cyl(232,242).
cyl(232,251).
cyl(233,244).
cyl(233,257).
cyl(234,256).
cyl(234,260).
cyl(235,262).
cyl(235,253).
cyl(236,259).
cyl(236,264).
cyl(237,261).
cyl(237,242).
cyl(238,260).
cyl(238,243).
cyl(239,260).
cyl(239,246).
cyl(240,254).
cyl(240,263).
cyl(241,265).
cyl(241,269).
cyl(242,283).
cyl(242,267).
cyl(243,270).
cyl(243,288).
cyl(244,280).
cyl(244,278).
cyl(245,271).
cyl(245,287).
cyl(246,284).
cyl(246,277).
cyl(247,288).
cyl(247,281).
cyl(248,280).
cyl(248,277).
cyl(249,273).
cyl(249,270).
cyl(250,277).
cyl(250,270).
cyl(251,286).
cyl(251,280).
cyl(252,279).
cyl(252,268).
cyl(253,283).
cyl(253,279).
cyl(254,277).
cyl(254,276).
cyl(255,265).
cyl(255,285).
cyl(256,277).
cyl(256,276).
cyl(257,284).
cyl(257,283).
cyl(258,270).
cyl(258,271).
cyl(259,277).
cyl(259,279).
cyl(260,284).
cyl(260,268).
cyl(261,267).
cyl(261,279).
cyl(262,271).
cyl(262,279).
cyl(263,268).
cyl(263,273).
cyl(264,272).
cyl(264,277).
cyl(265,297).
cyl(265,300).
cyl(266,302).
cyl(266,304).
cyl(267,292).
cyl(267,308).
cyl(268,296).
cyl(268,307).
cyl(269,306).
cyl(269,304).
cyl(270,300).
cyl(270,308).
cyl(271,293).
cyl(271,291).
cyl(272,294).
cyl(272,305).
cyl(273,293).
cyl(273,291).
cyl(274,303).
cyl(274,312).
cyl(275,294).
cyl(275,299).
cyl(276,292).
cyl(276,305).
cyl(277,303).
cyl(277,299).
cyl(278,297).
cyl(278,302).
cyl(279,302).
cyl(279,294).
cyl(280,291).
cyl(280,289).
cyl(281,294).
cyl(281,307).
cyl(282,293).
cyl(282,296).
cyl(283,308).
cyl(283,294).
cyl(284,302).
cyl(284,310).
cyl(285,289).
cyl(285,308).
cyl(286,292).
cyl(286,307).
cyl(287,293).
cyl(287,295).
cyl(288,296).
cyl(288,292).
cyl(289,322).
cyl(289,331).
cyl(290,333).
cyl(290,313).
cyl(291,326).
cyl(291,314).
cyl(292,334).
cyl(292,317).
cyl(293,317).
cyl(293,315).
cyl(294,333).
cyl(294,331).
cyl(295,321).
cyl(295,335).
cyl(296,314).
cyl(296,322).
cyl(297,321).
cyl(297,322).
cyl(298,332).
cyl(298,316).
cyl(299,321).
cyl(299,330).
cyl(300,320).
cyl(300,315).
cyl(301,317).
cyl(301,326).
cyl(302,335).
cyl(302,318).
cyl(303,336).
cyl(303,325).
cyl(304,325).
cyl(304,322).
cyl(305,332).
cyl(305,321).
cyl(306,335).
cyl(306,325).
cyl(307,323).
cyl(307,326).
cyl(308,316).
cyl(308,320).
cyl(309,321).
cyl(309,336).
cyl(310,322).
cyl(310,328).
cyl(311,332).
cyl(311,335).
cyl(312,332).
cyl(312,322).
cyl(313,359).
cyl(313,347).
cyl(314,348).
cyl(314,349).
cyl(315,350).
cyl(315,352).
cyl(316,351).
cyl(316,342).
cyl(317,354).
cyl(317,349).
cyl(318,340).
cyl(318,358).
cyl(319,359).
cyl(319,339).
cyl(320,357).
cyl(320,355).
cyl(321,357).
cyl(321,341).
cyl(322,344).
cyl(322,355).
cyl(323,340).
cyl(323,338).
cyl(324,342).
cyl(324,356).
cyl(325,355).
cyl(325,342).
cyl(326,345).
cyl(326,353).
cyl(327,345).
cyl(327,339).
cyl(328,360).
cyl(328,356).
cyl(329,358).
cyl(329,351).
cyl(330,359).
cyl(330,353).
cyl(331,341).
cyl(331,356).
cyl(332,344).
cyl(332,339).
cyl(333,351).
cyl(333,355).
cyl(334,355).
cyl(334,359).
cyl(335,350).
cyl(335,339).
cyl(336,342).
cyl(336,354).
cyl(337,365).
cyl(337,374).
cyl(338,364).
cyl(338,384).
cyl(339,373).
cyl(339,375).
cyl(340,380).
cyl(340,368).
cyl(341,372).
cyl(341,362).
cyl(342,368).
cyl(342,367).
cyl(343,364).
cyl(343,369).
cyl(344,382).
cyl(344,373).
cyl(345,367).
cyl(345,375).
cyl(346,370).
cyl(346,372).
cyl(347,363).
cyl(347,381).
cyl(348,371).
cyl(348,365).
cyl(349,372).
cyl(349,364).
cyl(350,379).
cyl(350,372).
cyl(351,381).
cyl(351,364).
cyl(352,381).
cyl(352,362).
cyl(353,370).
cyl(353,377).
cyl(354,373).
cyl(354,362).
cyl(355,367).
cyl(355,382).
cyl(356,370).
cyl(356,384).
cyl(357,371).
cyl(357,372).
cyl(358,361).
cyl(358,378).
cyl(359,371).
cyl(359,366).
cyl(360,382).
cyl(360,364).
cyl(361,407).
cyl(361,408).
cyl(362,392).
cyl(362,393).
cyl(363,393).
cyl(363,394).
cyl(364,387).
cyl(364,400).
cyl(365,397).
cyl(365,392).
cyl(366,400).
cyl(366,408).
cyl(367,401).
cyl(367,388).
cyl(368,389).
cyl(368,394).
cyl(369,388).
cyl(369,399).
cyl(370,405).
cyl(370,385).
cyl(371,398).
cyl(371,397).
cyl(372,404).
cyl(372,387).
cyl(373,404).
cyl(373,390).
cyl(374,396).
cyl(374,397).
cyl(375,401).
cyl(375,397).
cyl(376,399).
cyl(376,395).
cyl(377,397).
cyl(377,391).
cyl(378,392).
cyl(378,385).
cyl(379,390).
cyl(379,386).
cyl(380,408).
cyl(380,394).
cyl(381,398).
cyl(381,403).
cyl(382,385).
cyl(382,403).
cyl(383,385).
cyl(383,386).
cyl(384,397).
cyl(384,387).
cyl(385,418).
cyl(385,429).
cyl(386,419).
cyl(386,415).
cyl(387,413).
cyl(387,429).
cyl(388,415).
cyl(388,418).
cyl(389,429).
cyl(389,417).
cyl(390,417).
cyl(390,424).
cyl(391,409).
cyl(391,425).
cyl(392,418).
cyl(392,409).
cyl(393,428).
cyl(393,414).
cyl(394,427).
cyl(394,431).
cyl(395,429).
cyl(395,430).
cyl(396,418).
cyl(396,419).
cyl(397,432).
cyl(397,419).
cyl(398,420).
cyl(398,414).
cyl(399,419).
cyl(399,412).
cyl(400,415).
cyl(400,410).
cyl(401,420).
cyl(401,424).
cyl(402,426).
cyl(402,412).
cyl(403,431).
cyl(403,419).
cyl(404,428).
cyl(404,422).
cyl(405,417).
cyl(405,428).
cyl(406,422).
cyl(406,411).
cyl(407,424).
cyl(407,427).
cyl(408,410).
cyl(408,416).
cyl(409,436).
cyl(409,435).
cyl(410,442).
cyl(410,439).
cyl(411,456).
cyl(411,436).
cyl(412,449).
cyl(412,456).
cyl(413,453).
cyl(413,449).
cyl(414,440).
cyl(414,434).
cyl(415,436).
cyl(415,437).
cyl(416,433).
cyl(416,452).
cyl(417,433).
cyl(417,444).
cyl(418,436).
cyl(418,452).
cyl(419,445).
cyl(419,444).
cyl(420,451).
cyl(420,455).
cyl(421,439).
cyl(421,455).
cyl(422,445).
cyl(422,454).
cyl(423,456).
cyl(423,445).
cyl(424,445).
cyl(424,448).
cyl(425,434).
cyl(425,448).
cyl(426,442).
cyl(426,440).
cyl(427,437).
cyl(427,438).
cyl(428,453).
cyl(428,446).
cyl(429,437).
cyl(429,452).
cyl(430,444).
cyl(430,438).
cyl(431,449).
cyl(431,443).
cyl(432,442).
cyl(432,450).
cyl(433,469).
cyl(433,476).
cyl(434,476).
cyl(434,479).
cyl(435,478).
cyl(435,461).
cyl(436,467).
cyl(436,471).
cyl(437,479).
cyl(437,468).
cyl(438,474).
cyl(438,467).
cyl(439,459).
cyl(439,473).
cyl(440,458).
cyl(440,459).
cyl(441,467).
cyl(441,458).
cyl(442,470).
cyl(442,472).
cyl(443,477).
cyl(443,460).
cyl(444,475).
cyl(444,474).
cyl(445,471).
cyl(445,480).
cyl(446,477).
cyl(446,474).
cyl(447,472).
cyl(447,476).
cyl(448,469).
cyl(448,474).
cyl(449,465).
cyl(449,471).
cyl(450,465).
cyl(450,459).
cyl(451,458).
cyl(451,475).
cyl(452,457).
cyl(452,462).
cyl(453,478).
cyl(453,459).
cyl(454,472).
cyl(454,461).
cyl(455,469).
cyl(455,479).
cyl(456,457).
cyl(456,458).
cyl(457,482).
cyl(457,500).
cyl(458,492).
cyl(458,488).
cyl(459,488).
cyl(459,489).
cyl(460,483).
cyl(460,500).
cyl(461,504).
cyl(461,486).
cyl(462,491).
cyl(462,492).
cyl(463,499).
cyl(463,493).
cyl(464,483).
cyl(464,502).
cyl(465,495).
cyl(465,502).
cyl(466,483).
cyl(466,487).
cyl(467,491).
cyl(467,503).
cyl(468,492).
cyl(468,498).
cyl(469,501).
cyl(469,504).
cyl(470,484).
cyl(470,487).
cyl(471,502).
cyl(471,487).
cyl(472,499).
cyl(472,490).
cyl(473,500).
cyl(473,495).
cyl(474,481).
cyl(474,487).
cyl(475,488).
cyl(475,494).
cyl(476,488).
cyl(476,500).
cyl(477,492).
cyl(477,489).
cyl(478,504).
cyl(478,481).
cyl(479,502).
cyl(479,491).
cyl(480,497).
cyl(480,487).
cyl(481,528).
cyl(481,522).
cyl(482,522).
cyl(482,520).
cyl(483,516).
cyl(483,515).
cyl(484,526).
cyl(484,514).
cyl(485,511).
cyl(485,508).
cyl(486,512).
cyl(486,524).
cyl(487,525).
cyl(487,520).
cyl(488,508).
cyl(488,520).
cyl(489,526).
cyl(489,527).
cyl(490,517).
cyl(490,505).
cyl(491,514).
cyl(491,512).
cyl(492,524).
cyl(492,522).
cyl(493,524).
cyl(493,517).
cyl(494,520).
cyl(494,518).
cyl(495,516).
cyl(495,508).
cyl(496,508).
cyl(496,525).
cyl(497,523).
cyl(497,505).
cyl(498,507).
cyl(498,505).
cyl(499,510).
cyl(499,523).
cyl(500,522).
cyl(500,518).
cyl(501,511).
cyl(501,517).
cyl(502,506).
cyl(502,513).
cyl(503,505).
cyl(503,514).
cyl(504,525).
cyl(504,519).
cyl(505,547).
cyl(505,534).
cyl(506,551).
cyl(506,538).
cyl(507,538).
cyl(507,530).
cyl(508,551).
cyl(508,544).
cyl(509,550).
cyl(509,551).
cyl(510,529).
cyl(510,539).
cyl(511,544).
cyl(511,549).
cyl(512,543).
cyl(512,549).
cyl(513,540).
cyl(513,533).
cyl(514,551).
cyl(514,550).
cyl(515,536).
cyl(515,547).
cyl(516,544).
cyl(516,531).
cyl(517,535).
cyl(517,549).
cyl(518,546).
cyl(518,542).
cyl(519,537).
cyl(519,547).
cyl(520,547).
cyl(520,544).
cyl(521,531).
cyl(521,544).
cyl(522,533).
cyl(522,530).
cyl(523,538).
cyl(523,546).
cyl(524,541).
cyl(524,531).
cyl(525,530).
cyl(525,533).
cyl(526,530).
cyl(526,529).
cyl(527,550).
cyl(527,529).
cyl(528,541).
cyl(528,534).
cyl(529,564).
cyl(529,574).
cyl(530,554).
cyl(530,564).
cyl(531,564).
cyl(531,556).
cyl(532,569).
cyl(532,554).
cyl(533,561).
cyl(533,566).
cyl(534,565).
cyl(534,576).
cyl(535,570).
cyl(535,558).
cyl(536,572).
cyl(536,571).
cyl(537,555).
cyl(537,569).
cyl(538,564).
cyl(538,555).
cyl(539,558).
cyl(539,566).
cyl(540,571).
cyl(540,576).
cyl(541,567).
cyl(541,561).
cyl(542,573).
cyl(542,570).
cyl(543,576).
cyl(543,565).
cyl(544,572).
cyl(544,565).
cyl(545,553).
cyl(545,554).
cyl(546,556).
cyl(546,574).
cyl(547,553).
cyl(547,575).
cyl(548,571).
cyl(548,573).
cyl(549,556).
cyl(549,574).
cyl(550,575).
cyl(550,555).
cyl(551,558).
cyl(551,569).
cyl(552,569).
cyl(552,564)! !

!TestQueriesLayer methodsFor: 'slg tests' stamp: 'AndyKellens 11/3/2011 10:59'!
diffM1

tabled diff(?N) if 
	tabledpath(a,?N),not(tabledpath(c,?N))! !

!TestQueriesLayer methodsFor: 'slg tests' stamp: 'AndyKellens 11/3/2011 10:58'!
drpathM2

tabled drpath(?X,?Y) if 
	edge(?X,?Y).
tabled drpath(?X,?Y) if 
	drpath(?X,?Z),drpath(?Z,?Y)! !

!TestQueriesLayer methodsFor: 'slg tests' stamp: 'sp 24/10/11 15:26'!
edgeM2

edge(aa,ab).
edge(ab,ac).
edge(ac,ad).
edge(ad,ae).
edge(ae,af).
edge(af,ag).
edge(ag,ah).
edge(ah,ai).
edge(ai,aj).
edge(aj,ak).
edge(ak,al).
edge(al,am).
edge(am,an).
edge(an,ao).
edge(ao,ap).
edge(ap,aq).
edge(aq,ar).
edge(ar,as).
edge(as,at).
edge(at,au).
edge(au,av).
edge(av,aw).
edge(aw,ax).
edge(ax,ay).
edge(ay,az).
edge(az,ba).
edge(ba,bb).
edge(bb,bc).
edge(bc,bd).
edge(bd,be).
edge(be,bf).
edge(bf,bg).
edge(bg,bh).
edge(bh,bi).
edge(bi,bj).
edge(bj,bk).
edge(bk,bl).
edge(bl,bm).
edge(bm,bn).
edge(bn,bo).
edge(bo,bp).
edge(bp,bq).
edge(bq,br).
edge(br,bs).
edge(bs,bt).
edge(bt,bu).
edge(bu,bv).
edge(bv,bw).
edge(bw,bx).
edge(bx,by).
edge(by,bz).
edge(bz,ca).
edge(ca,cb).
edge(cb,cc).
edge(cc,cd).
edge(cd,ce).
edge(ce,cf).
edge(cf,cg).
edge(cg,ch).
edge(ch,ci).
edge(ci,cj).
edge(cj,ck).
edge(ck,cl).
edge(cl,cm).
edge(cm,cn).
edge(cn,co).
edge(co,cp).
edge(cp,cq).
edge(cq,cr).
edge(cr,cs).
edge(cs,ct).
edge(ct,cu).
edge(cu,cv).
edge(cv,cw).
edge(cw,cx).
edge(cx,cy).
edge(cy,cz).
edge(cz,da).
edge(da,db).
edge(db,dc).
edge(dc,dd).
edge(dd,de).
edge(de,df).
edge(df,dg).
edge(dg,dh).
edge(dh,di).
edge(di,dj).
edge(dj,dk).
edge(dk,dl).
edge(dl,dm).
edge(dm,dn).
edge(dn,do).
edge(do,dp).
edge(dp,dq).
edge(dq,dr).
edge(dr,ds).
edge(ds,dt).
edge(dt,du).
edge(du,dv).
edge(dv,dw).
edge(dw,dx).
edge(dx,dy).
edge(dy,dz).
edge(dz,ea).
edge(ea,eb).
edge(eb,ec).
edge(ec,ed).
edge(ed,ee).
edge(ee,ef).
edge(ef,eg).
edge(eg,eh).
edge(eh,ei).
edge(ei,ej).
edge(ej,ek).
edge(ek,el).
edge(el,em).
edge(em,en).
edge(en,eo).
edge(eo,ep).
edge(ep,eq).
edge(eq,er).
edge(er,es).
edge(es,et).
edge(et,eu).
edge(eu,ev).
edge(ev,ew).
edge(ew,ex).
edge(ex,ey).
edge(ey,ez).
edge(ez,fa).
edge(fa,fb).
edge(fb,fc).
edge(fc,fd).
edge(fd,fe).
edge(fe,ff).
edge(ff,fg).
edge(fg,fh).
edge(fh,fi).
edge(fi,fj).
edge(fj,fk).
edge(fk,fl).
edge(fl,fm).
edge(fm,fn).
edge(fn,fo).
edge(fo,fp).
edge(fp,fq).
edge(fq,fr).
edge(fr,fs).
edge(fs,ft).
edge(ft,fu).
edge(fu,fv).
edge(fv,fw).
edge(fw,fx).
edge(fx,fy).
edge(fy,fz).
edge(fz,ga).
edge(ga,gb).
edge(gb,gc).
edge(gc,gd).
edge(gd,ge).
edge(ge,gf).
edge(gf,gg).
edge(gg,gh).
edge(gh,gi).
edge(gi,gj).
edge(gj,gk).
edge(gk,gl).
edge(gl,gm).
edge(gm,gn).
edge(gn,go).
edge(go,gp).
edge(gp,gq).
edge(gq,gr).
edge(gr,gs).
edge(gs,gt).
edge(gt,gu).
edge(gu,gv).
edge(gv,gw).
edge(gw,gx).
edge(gx,gy).
edge(gy,gz).
edge(gz,ha).
edge(ha,hb).
edge(hb,hc).
edge(hc,hd).
edge(hd,he).
edge(he,hf).
edge(hf,hg).
edge(hg,hh).
edge(hh,hi).
edge(hi,hj).
edge(hj,hk).
edge(hk,hl).
edge(hl,hm).
edge(hm,hn).
edge(hn,ho).
edge(ho,hp).
edge(hp,hq).
edge(hq,hr).
edge(hr,hs).
edge(hs,ht).
edge(ht,hu).
edge(hu,hv).
edge(hv,hw).
edge(hw,hx).
edge(hx,hy).
edge(hy,hz).
edge(hz,ia).
edge(ia,ib).
edge(ib,ic).
edge(ic,id).
edge(id,ie).
edge(ie,ih).
edge(ih,ii).
edge(ii,ij).
edge(ij,ik).
edge(ik,il).
edge(il,im).
edge(im,in).
edge(in,io).
edge(io,ip).
edge(ip,iq).
edge(iq,ir).
edge(ir,is).
edge(is,it).
edge(it,iu).
edge(iu,iv).
edge(iv,iw).
edge(iw,ix).
edge(ix,iy).
edge(iy,iz).
edge(iz,ja).
edge(ja,jb).
edge(jb,jc).
edge(jc,jd).
edge(jd,je).
edge(je,jf).
edge(jf,jg).
edge(jg,jh).
edge(jh,ji).
edge(ji,jj).
edge(jj,jk).
edge(jk,jl).
edge(jl,jm).
edge(jm,jn).
edge(jn,jo).
edge(jo,jp).
edge(jp,jq).
edge(jq,jr).
edge(jr,js).
edge(js,jt).
edge(jt,ju).
edge(ju,jv).
edge(jv,jw).
edge(jw,jx).
edge(jx,jy).
edge(jy,jz).
edge(jz,ka).
edge(ka,kb).
edge(kb,kc).
edge(kc,kd).
edge(kd,ke).
edge(ke,kf).
edge(kf,kg).
edge(kg,kh).
edge(kh,ki).
edge(ki,kj).
edge(kj,kk).
edge(a,b).
edge(b,c).
edge(c,d)! !

!TestQueriesLayer methodsFor: 'slg tests' stamp: 'sp 24/10/11 15:26'!
manualpathM2

manualpath(?x,?y) if 
	slg(manualpath(?x,?y))! !

!TestQueriesLayer methodsFor: 'slg tests' stamp: 'sp 24/10/11 15:26'!
manualpathcont0M4

manualpathcont0(?Id,?vars,manualpath(?x,?z),D) if 
	edge(?z,?y),answer(?Id,manualpath(?x,?y),D)! !

!TestQueriesLayer methodsFor: 'slg tests' stamp: 'sp 24/10/11 15:26'!
moveM2

move(a,b).
move(b,c).
move(c,d).
move(b,a)! !

!TestQueriesLayer methodsFor: 'slg tests' stamp: 'AndyKellens 11/3/2011 11:00'!
mutRetAM2

tabled mutRetA(?X,?Y) if 
	mutRetD(?X,?Y).
tabled mutRetA(?X,?Y) if 
	mutRetB(?X,?Z),mutRetC(?Z,?Y)! !

!TestQueriesLayer methodsFor: 'slg tests' stamp: 'AndyKellens 11/3/2011 11:00'!
mutRetBM2

tabled mutRetB(?X,?Y) if 
	mutRetC(?X,?Y).
tabled mutRetB(?X,?Y) if 
	mutRetA(?X,?Z),mutRetD(?Z,?Y)! !

!TestQueriesLayer methodsFor: 'slg tests' stamp: 'sp 24/10/11 15:26'!
mutRetCM2

mutRetC(2,2).
mutRetC(3,3)! !

!TestQueriesLayer methodsFor: 'slg tests' stamp: 'sp 24/10/11 15:26'!
mutRetDM2

mutRetD(0,1).
mutRetD(1,2).
mutRetD(2,3)! !

!TestQueriesLayer methodsFor: 'slg tests' stamp: 'sp 24/10/11 15:26'!
pathM2

path(?x,?y) if 
	edge(?x,?y).
path(?x,?y) if 
	edge(?x,?z),path(?z,?y)! !

!TestQueriesLayer methodsFor: 'slg tests' stamp: 'sp 24/10/11 15:26'!
pathcont0M2

pathcont0(?Id,path(?x,?z)) if 
	edge(?z,?y),answer(?Id,path(?x,?y),?D)! !

!TestQueriesLayer methodsFor: 'ctl' stamp: 'sp 24/10/11 15:26'!
propM2

prop(s0,p).
prop(s0,q).
prop(s1,q).
prop(s1,r).
prop(s2,r)! !

!TestQueriesLayer methodsFor: 'slg performance' stamp: 'sp 24/10/11 15:26'!
sameGenerationM2

sameGeneration(?X,?X) if 
	[true].
sameGeneration(?X,?Y) if 
	cyl(?X,?Z),sameGeneration(?Z,?Z),cyl(?Y,?Z)! !

!TestQueriesLayer methodsFor: 'slg tests' stamp: 'AndyKellens 11/3/2011 11:11'!
shortedgeM2

shortedge(a,b).
shortedge(b,c).
shortedge(c,d). 
shortedge(d,a)! !

!TestQueriesLayer methodsFor: 'slg tests' stamp: 'AndyKellens 11/3/2011 11:03'!
shortpathM2

tabled shortpath(?X,?Y) if 
	shortedge(?X,?Y).
tabled shortpath(?x,?y) if 
	shortpath(?x,?z),shortedge(?z,?y)! !

!TestQueriesLayer methodsFor: 'slg tests' stamp: 'sp 24/10/11 15:26'!
slgmanualpathM3

slgmanualpath(?x,?y,?Id) if 
	slgcall(?Id,?vars,manualpath(?x,?z),0,manualpathcont0).
slgmanualpath(?x,?y,?Id) if 
	edge(?x,?y),answer(?Id,manualpath(?x,?y),0)! !

!TestQueriesLayer methodsFor: 'slg tests' stamp: 'AndyKellens 11/3/2011 11:00'!
smalltalkPathXM2

tabled ?x smalltalkPath: ?y if 
	edge(?x,?y).
tabled ?x smalltalkPath: ?y if 
	edge(?x,?z),?z smalltalkPath: ?y! !

!TestQueriesLayer methodsFor: 'slg tests' stamp: 'AndyKellens 10/28/2011 13:07'!
tabledpathM2

tabled tabledpath(?x,?y) if 
	tabledpath(?x,?z),edge(?z,?y).
tabled tabledpath(?x,?y) if 
	edge(?x,?y)! !

!TestQueriesLayer methodsFor: 'regexqueries' stamp: 'sp 24/10/11 15:26'!
targetM2

target(?A,?E) if 
	?A equals: ?E! !

!TestQueriesLayer methodsFor: 'ctl' stamp: 'sp 24/10/11 15:26'!
transM2

trans(s1,s1).
trans(s0,s2).
trans(s2,s0).
trans(s2,s1).
trans(s0,s1).
"prevent our testqueries from messing with the versionmodel"

trans(?X,?Y) if 
	!!,[false]! !

!TestQueriesLayer methodsFor: 'regexqueries' stamp: 'sp 24/10/11 15:26'!
transM3

trans(s0,s1,a).
trans(s1,s2,b).
trans(s2,s5,a).
trans(s0,s3,a).
trans(s3,s4,b).
trans(s4,s5,a).
trans(s4,s5,b).
trans(s2,s5,b)! !

!TestQueriesLayer methodsFor: 'slg tests' stamp: 'AndyKellens 11/3/2011 10:59'!
winM1

tabled win(?X) if 
	move(?X,?Y),not(win(?Y))! !
LogicProgramControlTest initialize!
